@@ -39,9 +39,6 @@
 
 extern PyTypeObject PyJBuffer_Type;
 
-#define PyJBuffer_Wrap(env, jobj, jcls) \
-    PyJObject_New(env, &PyJBuffer_Type, jobj, jcls)
-
 #define PyJBuffer_Check(pyobj) \
     PyObject_TypeCheck(pyobj, &PyJBuffer_Type)
 
