@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2015-2019 JEP AUTHORS.
+   Copyright (c) 2015-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -38,9 +38,6 @@
 
 extern PyTypeObject PyJIterator_Type;
 
-#define PyJIterator_Wrap(env, jobj, jcls) \
-    PyJObject_New(env, &PyJIterator_Type, jobj, jcls)
-
 #define PyJIterator_Check(pyobj) \
     PyObject_TypeCheck(pyobj, &PyJIterator_Type)
 
