@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2015-2019 JEP AUTHORS.
+   Copyright (c) 2015-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -39,9 +39,6 @@
 
 extern PyTypeObject PyJList_Type;
 
-#define PyJList_Wrap(env, jobj, jcls) \
-    PyJObject_New(env, &PyJList_Type, jobj, jcls)
-
 #define PyJList_Check(pyobj) \
     PyObject_TypeCheck(pyobj, &PyJList_Type)
 
