@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2016-2019 JEP AUTHORS.
+   Copyright (c) 2016-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -39,9 +39,6 @@
 
 extern PyTypeObject PyJNumber_Type;
 
-#define PyJNumber_Wrap(env, jobj, jcls) \
-    PyJObject_New(env, &PyJNumber_Type, jobj, jcls)
-
 #define PyJNumber_Check(pyobj) \
     PyObject_TypeCheck(pyobj, &PyJNumber_Type)
 
