@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2016-2019 JEP AUTHORS.
+   Copyright (c) 2016-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -40,10 +40,6 @@
 
 #define JCHAR_MAX   0xFFFF
 
-#if PY_MAJOR_VERSION < 3
-    static jstring UTF8 = NULL;
-#endif
-
 /*
  * When there is no way to convert a PyObject of a specific expected Java type
  * then a Python TypeError is raised. This function is used to make a pretty
@@ -63,7 +59,7 @@ static void raiseTypeError(JNIEnv *env, PyObject *pyobject, jclass expectedType)
     if (PyJClass_Check(pyobject)) {
         actTypeName = "java.lang.Class";
     } else if (PyJObject_Check(pyobject)) {
-        actTypeName = PyString_AsString(((PyJObject*) pyobject)->javaClassName);
+        actTypeName = PyUnicode_AsUTF8(((PyJObject*) pyobject)->javaClassName);
     } else {
         actTypeName = pyobject->ob_type->tp_name;
     }
@@ -201,14 +197,6 @@ jdouble PyObject_As_jdouble(PyObject *pyfloat)
 
 static jchar pyunicode_as_jchar(PyObject *pyunicode)
 {
-#if PY_MAJOR_VERSION < 3
-    if (PyUnicode_Check(pyunicode) && PyUnicode_GET_SIZE(pyunicode) == 1) {
-        Py_UNICODE* data = PyUnicode_AS_UNICODE(pyunicode);
-        if (data[0] < JCHAR_MAX) {
-            return (jchar) data[0];
-        }
-    }
-#else
     if (PyUnicode_Check(pyunicode)) {
         if (PyUnicode_READY(pyunicode) != 0) {
             return 0;
@@ -220,7 +208,6 @@ static jchar pyunicode_as_jchar(PyObject *pyunicode)
             }
         }
     }
-#endif
     PyErr_Format(PyExc_TypeError, "Expected char but received a %s.",
                  pyunicode->ob_type->tp_name);
     return 0;
@@ -231,7 +218,6 @@ static jstring pyunicode_as_jstring(JNIEnv *env, PyObject *pyunicode)
 {
     PyObject* bytes = NULL;
     jstring result  = NULL;
-#if PY_MAJOR_VERSION >= 3
     if (PyUnicode_READY(pyunicode) != 0) {
         return NULL;
     } else if (PyUnicode_KIND((pyunicode)) == PyUnicode_2BYTE_KIND) {
@@ -239,7 +225,6 @@ static jstring pyunicode_as_jstring(JNIEnv *env, PyObject *pyunicode)
         Py_ssize_t length = PyUnicode_GET_LENGTH(pyunicode);
         return (*env)->NewString(env, (jchar*) data, (jsize) length);
     }
-#endif
     bytes = PyUnicode_AsUTF16String(pyunicode);
     if (bytes == NULL) {
         return NULL;
@@ -251,102 +236,9 @@ static jstring pyunicode_as_jstring(JNIEnv *env, PyObject *pyunicode)
     return result;
 }
 
-
-#if PY_MAJOR_VERSION < 3
-static jchar pystring_as_jchar(PyObject *pystring)
-{
-    if (PyString_Check(pystring)) {
-        if (PyString_Size(pystring) == 1) {
-            return (unsigned char) PyString_AsString(pystring)[0];
-        } else if (PyString_Size(pystring) < 4) {
-            PyObject* pyunicode = PyUnicode_DecodeUTF8(PyString_AsString(pystring),
-                                  PyString_Size(pystring), NULL);
-            if (PyUnicode_GET_SIZE(pyunicode) == 1) {
-                Py_UNICODE* data = PyUnicode_AS_UNICODE(pyunicode);
-                if (data[0] < JCHAR_MAX) {
-                    jchar result = (jchar) data[0];
-                    Py_DECREF(pyunicode);
-                    return result;
-                }
-            }
-            Py_DECREF(pyunicode);
-        }
-    }
-    PyErr_Format(PyExc_TypeError, "Expected char but received a %s.",
-                 pystring->ob_type->tp_name);
-    return 0;
-}
-
-
-static jstring pystring_as_jstring(JNIEnv *env, PyObject *pystring)
-{
-    // Do not use NewStringUTF because it does not expext true UTF-8 and fails
-    // for some unicode input
-    char*      cstr         = NULL;
-    Py_ssize_t length       = 0;
-    jbyteArray stringJbytes = NULL;
-    jstring result          = NULL;
-    if ( UTF8 == NULL) {
-        jobject local = (*env)->NewStringUTF(env, "UTF-8");
-        UTF8 = (*env)->NewGlobalRef(env, local);
-        (*env)->DeleteLocalRef(env, local);
-    }
-    cstr = PyString_AsString(pystring);
-    if (cstr == NULL) {
-        return NULL;
-    }
-
-    length = PyString_Size(pystring);
-    stringJbytes = (*env)->NewByteArray(env, (jsize) length);
-    if (process_java_exception(env)) {
-        return NULL;
-    }
-
-    (*env)->SetByteArrayRegion(env, stringJbytes, 0, (jsize) length, (jbyte*) cstr);
-    result = java_lang_String_new_BArray_String(env, stringJbytes, UTF8);
-    if (process_java_exception(env)) {
-        return NULL;
-    }
-    (*env)->DeleteLocalRef(env, stringJbytes);
-    return result;
-}
-
-static jobject pystring_as_jobject(JNIEnv *env, PyObject *pyobject,
-                                   jclass expectedType)
-{
-    if ((*env)->IsAssignableFrom(env, JSTRING_TYPE, expectedType)) {
-        return (jobject) pystring_as_jstring(env, pyobject);
-    } else if ((*env)->IsAssignableFrom(env, JCHAR_OBJ_TYPE, expectedType)) {
-        jobject result;
-        jchar c = pystring_as_jchar(pyobject);
-        if (c == 0 && PyErr_Occurred()) {
-            return NULL;
-        }
-        result = java_lang_Character_new_C(env, c);
-        if (!result) {
-            process_java_exception(env);
-            return NULL;
-        }
-        return result;
-    } else if ((*env)->IsAssignableFrom(env, JPYOBJECT_TYPE, expectedType)) {
-        return PyObject_As_JPyObject(env, pyobject);
-    }
-    raiseTypeError(env, pyobject, expectedType);
-    return NULL;
-}
-#endif
-
 jchar PyObject_As_jchar(PyObject *pyobject)
 {
-#if PY_MAJOR_VERSION < 3
-    if (PyUnicode_Check(pyobject)) {
-        return pyunicode_as_jchar(pyobject);
-    } else {
-        return pystring_as_jchar(pyobject);
-    }
-#else
     return pyunicode_as_jchar(pyobject);
-#endif
 }
 
 
@@ -358,15 +250,7 @@ jstring PyObject_As_jstring(JNIEnv *env, PyObject *pyobject)
     if (pystring == NULL) {
         return NULL;
     }
-#if PY_MAJOR_VERSION < 3
-    if (PyUnicode_Check(pystring)) {
-        result =  pyunicode_as_jstring(env, pystring);
-    } else {
-        result = pystring_as_jstring(env, pystring);
-    }
-#else
     result = pyunicode_as_jstring(env, pystring);
-#endif
     Py_DECREF(pystring);
     return result;
 }
@@ -417,6 +301,21 @@ static jobject pyunicode_as_jobject(JNIEnv *env, PyObject *pyobject,
     return NULL;
 }
 
+static jobject pylong_as_jbiginteger(JNIEnv *env, PyObject *pyobject,
+                                 jclass expectedType)
+{
+    jstring s = PyObject_As_jstring(env, pyobject);
+    if (s == NULL) {
+        return NULL;
+    }
+    jobject result = java_math_BigInteger_new_String(env, s);
+    if (!result) {
+        process_java_exception(env);
+        return NULL;
+    }
+    return result;
+}
+
 static jobject pylong_as_jobject(JNIEnv *env, PyObject *pyobject,
                                  jclass expectedType)
 {
@@ -424,6 +323,11 @@ static jobject pylong_as_jobject(JNIEnv *env, PyObject *pyobject,
         jobject result;
         jlong j = PyObject_As_jlong(pyobject);
         if (j == -1 && PyErr_Occurred()) {
+            if (PyErr_ExceptionMatches(PyExc_OverflowError) &&
+                (*env)->IsAssignableFrom(env, JBIGINTEGER_TYPE, expectedType)) {
+                PyErr_Clear();
+                return pylong_as_jbiginteger(env, pyobject, expectedType);
+            }
             return NULL;
         }
         result = java_lang_Long_new_J(env, j);
@@ -470,38 +374,13 @@ static jobject pylong_as_jobject(JNIEnv *env, PyObject *pyobject,
         return result;
     } else if ((*env)->IsAssignableFrom(env, JPYOBJECT_TYPE, expectedType)) {
         return PyObject_As_JPyObject(env, pyobject);
+    } else if ((*env)->IsAssignableFrom(env, JBIGINTEGER_TYPE, expectedType)) {
+        return pylong_as_jbiginteger(env, pyobject, expectedType);
     }
     raiseTypeError(env, pyobject, expectedType);
     return NULL;
 }
 
-#if PY_MAJOR_VERSION < 3
-static jobject pyint_as_jobject(JNIEnv *env, PyObject *pyobject,
-                                jclass expectedType)
-{
-    if ((*env)->IsAssignableFrom(env, JINT_OBJ_TYPE, expectedType)) {
-        jobject result;
-        jint i = PyObject_As_jint(pyobject);
-        if (i == -1 && PyErr_Occurred()) {
-            if (PyErr_ExceptionMatches(PyExc_OverflowError)) {
-                /* Just in case expectedType is Object or Number and it fits in a long. */
-                PyErr_Clear();
-                return pylong_as_jobject(env, pyobject, expectedType);
-            } else {
-                return NULL;
-            }
-        }
-        result = java_lang_Integer_new_I(env, i);
-        if (!result) {
-            process_java_exception(env);
-            return NULL;
-        }
-        return result;
-    }
-    return pylong_as_jobject(env, pyobject, expectedType);
-}
-#endif
-
 static jobject pyfloat_as_jobject(JNIEnv *env, PyObject *pyobject,
                                   jclass expectedType)
 {
@@ -845,31 +724,11 @@ static jobject PyCallable_As_JPyCallable(JNIEnv *env, PyObject *pyobject)
     return jpycallable;
 }
 
-static jbyteArray pybuffer_as_jbytearray(JNIEnv *env, PyObject *pybuffer)
+static jbyteArray pybufferview_as_jbytearray(JNIEnv *env, Py_buffer view)
 {
-    Py_ssize_t length = 0;
-    jbyteArray jarr = NULL;
-    Py_buffer view;
-
-    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
-        return NULL;
-    }
-    if (!(view.format == NULL || strcmp(view.format, "b") == 0
-            || strcmp(view.format, "B") == 0)
-            || view.itemsize != sizeof(jbyte) ) {
-        const char *format = view.format;
-        if (format == NULL) {
-            format = "B";
-        }
-        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a byte[].",
-                     view.format);
-        PyBuffer_Release(&view);
-        return NULL;
-    }
-    length = view.len / view.itemsize;
-    jarr = (*env)->NewByteArray(env, (jsize) length);
+    Py_ssize_t length = view.len / view.itemsize;
+    jbyteArray jarr = (*env)->NewByteArray(env, (jsize) length);
     if (process_java_exception(env)) {
-        PyBuffer_Release(&view);
         return NULL;
     }
     if (PyBuffer_IsContiguous(&view, 'A')) {
@@ -879,34 +738,44 @@ static jbyteArray pybuffer_as_jbytearray(JNIEnv *env, PyObject *pybuffer)
         PyBuffer_ToContiguous(buf, &view, view.len, 'C');
         (*env)->ReleaseByteArrayElements(env, jarr, buf, 0);
     }
-    PyBuffer_Release(&view);
     return jarr;
 }
 
-static jshortArray pybuffer_as_jshortarray(JNIEnv *env, PyObject *pybuffer)
+static int pybufferview_check_jbytearray(Py_buffer view)
 {
-    Py_ssize_t length = 0;
-    jshortArray jarr = NULL;
+    return (view.format == NULL || strcmp(view.format, "b") == 0
+            || strcmp(view.format, "B") == 0)
+           && view.itemsize == sizeof(jbyte);
+}
+
+static jbyteArray pybuffer_as_jbytearray(JNIEnv *env, PyObject *pybuffer)
+{
+    jbyteArray jarr = NULL;
     Py_buffer view;
 
-    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL) < 0) {
+    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
         return NULL;
     }
-    if (view.format == NULL || strcmp(view.format, "h") != 0
-            || view.itemsize != sizeof(jshort) ) {
+    if ( pybufferview_check_jbytearray(view) ) {
+        jarr = pybufferview_as_jbytearray(env, view);
+    } else {
         const char *format = view.format;
         if (format == NULL) {
             format = "B";
         }
-        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a short[].",
+        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a byte[].",
                      view.format);
-        PyBuffer_Release(&view);
-        return NULL;
     }
-    length = view.len / view.itemsize;
-    jarr = (*env)->NewShortArray(env, (jsize) length);
+
+    PyBuffer_Release(&view);
+    return jarr;
+}
+
+static jshortArray pybufferview_as_jshortarray(JNIEnv *env, Py_buffer view)
+{
+    Py_ssize_t length = view.len / view.itemsize;
+    jshortArray jarr = (*env)->NewShortArray(env, (jsize) length);
     if (process_java_exception(env)) {
-        PyBuffer_Release(&view);
         return NULL;
     }
     if (PyBuffer_IsContiguous(&view, 'A')) {
@@ -916,35 +785,43 @@ static jshortArray pybuffer_as_jshortarray(JNIEnv *env, PyObject *pybuffer)
         PyBuffer_ToContiguous(buf, &view, view.len, 'C');
         (*env)->ReleaseShortArrayElements(env, jarr, buf, 0);
     }
-    PyBuffer_Release(&view);
     return jarr;
 }
 
-static jintArray pybuffer_as_jintarray(JNIEnv *env, PyObject *pybuffer)
+static int pybufferview_check_jshortarray(Py_buffer view)
 {
-    Py_ssize_t length = 0;
-    jintArray jarr = NULL;
+    return view.format != NULL && strcmp(view.format, "h") == 0
+           && view.itemsize == sizeof(jshort);
+}
+
+static jshortArray pybuffer_as_jshortarray(JNIEnv *env, PyObject *pybuffer)
+{
+    jshortArray jarr = NULL;
     Py_buffer view;
 
-    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL) < 0) {
+    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
         return NULL;
     }
-    if (view.format == NULL
-            || !(strcmp(view.format, "i") == 0 || strcmp(view.format, "l") == 0)
-            || view.itemsize != sizeof(jint) ) {
+    if ( pybufferview_check_jshortarray(view) ) {
+        jarr = pybufferview_as_jshortarray(env, view);
+    } else {
         const char *format = view.format;
         if (format == NULL) {
             format = "B";
         }
-        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a int[].",
+        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a short[].",
                      view.format);
-        PyBuffer_Release(&view);
-        return NULL;
     }
-    length = view.len / view.itemsize;
-    jarr = (*env)->NewIntArray(env, (jsize) length);
+
+    PyBuffer_Release(&view);
+    return jarr;
+}
+
+static jintArray pybufferview_as_jintarray(JNIEnv *env, Py_buffer view)
+{
+    Py_ssize_t length = view.len / view.itemsize;
+    jintArray jarr = (*env)->NewIntArray(env, (jsize) length);
     if (process_java_exception(env)) {
-        PyBuffer_Release(&view);
         return NULL;
     }
     if (PyBuffer_IsContiguous(&view, 'A')) {
@@ -954,35 +831,43 @@ static jintArray pybuffer_as_jintarray(JNIEnv *env, PyObject *pybuffer)
         PyBuffer_ToContiguous(buf, &view, view.len, 'C');
         (*env)->ReleaseIntArrayElements(env, jarr, buf, 0);
     }
-    PyBuffer_Release(&view);
     return jarr;
 }
 
-static jlongArray pybuffer_as_jlongarray(JNIEnv *env, PyObject *pybuffer)
+static int pybufferview_check_jintarray(Py_buffer view)
 {
-    Py_ssize_t length = 0;
-    jlongArray jarr = NULL;
+    return view.format != NULL
+           && (strcmp(view.format, "i") == 0 || strcmp(view.format, "l") == 0)
+           && view.itemsize == sizeof(jint);
+}
+
+static jintArray pybuffer_as_jintarray(JNIEnv *env, PyObject *pybuffer)
+{
+    jintArray jarr = NULL;
     Py_buffer view;
 
-    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL) < 0) {
+    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
         return NULL;
     }
-    if (view.format == NULL
-            || !(strcmp(view.format, "l") == 0 || strcmp(view.format, "q") == 0)
-            || view.itemsize != sizeof(jlong) ) {
+    if (pybufferview_check_jintarray(view)) {
+        jarr = pybufferview_as_jintarray(env, view);
+    } else {
         const char *format = view.format;
         if (format == NULL) {
             format = "B";
         }
-        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a long[].",
+        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a int[].",
                      view.format);
-        PyBuffer_Release(&view);
-        return NULL;
     }
-    length = view.len / view.itemsize;
-    jarr = (*env)->NewLongArray(env, (jsize) length);
+    PyBuffer_Release(&view);
+    return jarr;
+}
+
+static jlongArray pybufferview_as_jlongarray(JNIEnv *env, Py_buffer view)
+{
+    Py_ssize_t length = view.len / view.itemsize;
+    jlongArray jarr = (*env)->NewLongArray(env, (jsize) length);
     if (process_java_exception(env)) {
-        PyBuffer_Release(&view);
         return NULL;
     }
     if (PyBuffer_IsContiguous(&view, 'A')) {
@@ -992,34 +877,44 @@ static jlongArray pybuffer_as_jlongarray(JNIEnv *env, PyObject *pybuffer)
         PyBuffer_ToContiguous(buf, &view, view.len, 'C');
         (*env)->ReleaseLongArrayElements(env, jarr, buf, 0);
     }
-    PyBuffer_Release(&view);
     return jarr;
 }
 
-static jfloatArray pybuffer_as_jfloatarray(JNIEnv *env, PyObject *pybuffer)
+static int pybufferview_check_jlongarray(Py_buffer view)
 {
-    Py_ssize_t length = 0;
-    jfloatArray jarr = NULL;
+    return view.format != NULL
+           &&  (strcmp(view.format, "l") == 0 || strcmp(view.format, "q") == 0)
+           && view.itemsize == sizeof(jlong);
+}
+
+static jlongArray pybuffer_as_jlongarray(JNIEnv *env, PyObject *pybuffer)
+{
+    jlongArray jarr = NULL;
     Py_buffer view;
 
-    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL) < 0) {
+    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
         return NULL;
     }
-    if (view.format == NULL || strcmp(view.format, "f") != 0
-            || view.itemsize != sizeof(jfloat) ) {
+    if ( pybufferview_check_jlongarray(view) ) {
+        jarr = pybufferview_as_jlongarray(env, view);
+    } else {
         const char *format = view.format;
         if (format == NULL) {
             format = "B";
         }
-        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a float[].",
+        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a long[].",
                      view.format);
-        PyBuffer_Release(&view);
-        return NULL;
     }
-    length = view.len / view.itemsize;
-    jarr = (*env)->NewFloatArray(env, (jsize) length);
+
+    PyBuffer_Release(&view);
+    return jarr;
+}
+
+static jfloatArray pybufferview_as_jfloatarray(JNIEnv *env, Py_buffer view)
+{
+    Py_ssize_t length = view.len / view.itemsize;
+    jfloatArray jarr = (*env)->NewFloatArray(env, (jsize) length);
     if (process_java_exception(env)) {
-        PyBuffer_Release(&view);
         return NULL;
     }
     if (PyBuffer_IsContiguous(&view, 'A')) {
@@ -1029,34 +924,42 @@ static jfloatArray pybuffer_as_jfloatarray(JNIEnv *env, PyObject *pybuffer)
         PyBuffer_ToContiguous(buf, &view, view.len, 'C');
         (*env)->ReleaseFloatArrayElements(env, jarr, buf, 0);
     }
-    PyBuffer_Release(&view);
     return jarr;
 }
 
-static jdoubleArray pybuffer_as_jdoublearray(JNIEnv *env, PyObject *pybuffer)
+static int pybufferview_check_jfloatarray(Py_buffer view)
 {
-    Py_ssize_t length = 0;
-    jdoubleArray jarr = NULL;
+    return view.format != NULL && strcmp(view.format, "f") == 0
+           && view.itemsize == sizeof(jfloat);
+}
+
+static jfloatArray pybuffer_as_jfloatarray(JNIEnv *env, PyObject *pybuffer)
+{
+    jfloatArray jarr = NULL;
     Py_buffer view;
 
-    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL) < 0) {
+    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
         return NULL;
     }
-    if (view.format == NULL || strcmp(view.format, "d") != 0
-            || view.itemsize != sizeof(jdouble) ) {
+    if (pybufferview_check_jfloatarray(view)) {
+        jarr = pybufferview_as_jfloatarray(env, view);
+    } else {
         const char *format = view.format;
         if (format == NULL) {
             format = "B";
         }
-        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a double[].",
+        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a float[].",
                      view.format);
-        PyBuffer_Release(&view);
-        return NULL;
     }
-    length = view.len / view.itemsize;
-    jarr = (*env)->NewDoubleArray(env, (jsize) length);
+    PyBuffer_Release(&view);
+    return jarr;
+}
+
+static jdoubleArray pybufferview_as_jdoublearray(JNIEnv *env, Py_buffer view)
+{
+    Py_ssize_t length = view.len / view.itemsize;
+    jdoubleArray jarr = (*env)->NewDoubleArray(env, (jsize) length);
     if (process_java_exception(env)) {
-        PyBuffer_Release(&view);
         return NULL;
     }
     if (PyBuffer_IsContiguous(&view, 'A')) {
@@ -1066,6 +969,33 @@ static jdoubleArray pybuffer_as_jdoublearray(JNIEnv *env, PyObject *pybuffer)
         PyBuffer_ToContiguous(buf, &view, view.len, 'C');
         (*env)->ReleaseDoubleArrayElements(env, jarr, buf, 0);
     }
+    return jarr;
+}
+
+static int pybufferview_check_jdoublearray(Py_buffer view)
+{
+    return view.format != NULL && strcmp(view.format, "d") == 0
+           && view.itemsize == sizeof(jdouble);
+}
+
+static jdoubleArray pybuffer_as_jdoublearray(JNIEnv *env, PyObject *pybuffer)
+{
+    jdoubleArray jarr = NULL;
+    Py_buffer view;
+
+    if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
+        return NULL;
+    }
+    if ( pybufferview_check_jdoublearray(view) ) {
+        jarr = pybufferview_as_jdoublearray(env, view);
+    } else {
+        const char *format = view.format;
+        if (format == NULL) {
+            format = "B";
+        }
+        PyErr_Format(PyExc_TypeError, "Buffer format '%s' is not valid for a double[].",
+                     view.format);
+    }
     PyBuffer_Release(&view);
     return jarr;
 }
@@ -1073,7 +1003,34 @@ static jdoubleArray pybuffer_as_jdoublearray(JNIEnv *env, PyObject *pybuffer)
 static jobject pybuffer_as_jobject(JNIEnv *env, PyObject *pybuffer,
                                    jclass expectedType)
 {
-    if ((*env)->IsAssignableFrom(env, JBYTE_ARRAY_TYPE, expectedType)) {
+    if ((*env)->IsSameObject(env, JOBJECT_TYPE, expectedType)) {
+        /*
+         * Expect any java.lang.Object so must access the view to determine
+         * what type of java array is compatible
+         */
+        Py_buffer view;
+        jobject result = NULL;
+        if (PyObject_GetBuffer(pybuffer, &view, PyBUF_FULL_RO) < 0) {
+            return NULL;
+        }
+        if (pybufferview_check_jbytearray(view)) {
+            result = (jobject) pybufferview_as_jbytearray(env, view);
+        } else if (pybufferview_check_jshortarray(view)) {
+            result = (jobject) pybufferview_as_jshortarray(env, view);
+        } else if (pybufferview_check_jintarray(view)) {
+            result = (jobject) pybufferview_as_jintarray(env, view);
+        } else if (pybufferview_check_jlongarray(view)) {
+            result = (jobject) pybufferview_as_jlongarray(env, view);
+        } else if (pybufferview_check_jfloatarray(view)) {
+            result = (jobject) pybufferview_as_jfloatarray(env, view);
+        } else if (pybufferview_check_jdoublearray(view)) {
+            result = (jobject) pybufferview_as_jdoublearray(env, view);
+        } else {
+            result = PyObject_As_JPyObject(env, pybuffer);
+        }
+        PyBuffer_Release(&view);
+        return result;
+    } else if ((*env)->IsAssignableFrom(env, JBYTE_ARRAY_TYPE, expectedType)) {
         return (jobject) pybuffer_as_jbytearray(env, pybuffer);
     } else if ((*env)->IsAssignableFrom(env, JINT_ARRAY_TYPE, expectedType)) {
         return (jobject) pybuffer_as_jintarray(env, pybuffer);
@@ -1114,10 +1071,6 @@ jobject PyObject_As_jobject(JNIEnv *env, PyObject *pyobject,
         }
     } else if ((*env)->IsSameObject(env, expectedType, JPYOBJECT_TYPE)) {
         return PyObject_As_JPyObject(env, pyobject);
-#if PY_MAJOR_VERSION < 3
-    } else if (PyString_Check(pyobject)) {
-        return pystring_as_jobject(env, pyobject, expectedType);
-#endif
     } else if (PyUnicode_Check(pyobject)) {
         return pyunicode_as_jobject(env, pyobject, expectedType);
     } else if (PyBool_Check(pyobject)) {
@@ -1133,10 +1086,6 @@ jobject PyObject_As_jobject(JNIEnv *env, PyObject *pyobject,
 #endif
     } else if (PyLong_Check(pyobject)) {
         return pylong_as_jobject(env, pyobject, expectedType);
-#if PY_MAJOR_VERSION < 3
-    } else if (PyInt_Check(pyobject)) {
-        return pyint_as_jobject(env, pyobject, expectedType);
-#endif
     } else if (PyFloat_Check(pyobject)) {
         return pyfloat_as_jobject(env, pyobject, expectedType);
     } else if (PyList_Check(pyobject) || PyTuple_Check(pyobject)) {
@@ -1157,10 +1106,10 @@ jobject PyObject_As_jobject(JNIEnv *env, PyObject *pyobject,
 #endif
     } else if (PyObject_CheckBuffer(pyobject)) {
         return pybuffer_as_jobject(env, pyobject, expectedType);
-    } else if ((*env)->IsAssignableFrom(env, JSTRING_TYPE, expectedType)) {
-        return (jobject) PyObject_As_jstring(env, pyobject);
     } else if ((*env)->IsAssignableFrom(env, JPYOBJECT_TYPE, expectedType)) {
         return PyObject_As_JPyObject(env, pyobject);
+    } else if ((*env)->IsAssignableFrom(env, JSTRING_TYPE, expectedType)) {
+        return (jobject) PyObject_As_jstring(env, pyobject);
     }
     raiseTypeError(env, pyobject, expectedType);
     return NULL;
