@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2004-2019 JEP AUTHORS.
+   Copyright (c) 2004-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -52,16 +52,6 @@ int process_py_exception(JNIEnv *env)
         return 0;
     }
 
-    // let's not turn this into a Java exception if the user exited
-    if (PyErr_ExceptionMatches(PyExc_SystemExit)) {
-        /*
-         * If you look at the CPython source code, the following line will
-         * trigger a clean exit from Python.
-         */
-        PyErr_PrintEx(1);
-        return 0;
-    }
-
     if ((*env)->ExceptionCheck(env)) {
         /*
          * There's a bug in Jep somewhere, hopefully this printout will help
@@ -82,6 +72,34 @@ int process_py_exception(JNIEnv *env)
     PyErr_Fetch(&ptype, &pvalue, &ptrace);
 
     if (ptype) {
+        // let's not turn this into a Java exception if the user exited
+        if (ptype == PyExc_SystemExit) {
+            // If the exception is not normalized the exit code is in pvalue.
+            PyObject *code = pvalue;
+            if (PyErr_GivenExceptionMatches(pvalue, ptype)) {
+                // If the exception is normalized the exit code is in the code attribute.
+                code = PyObject_GetAttrString(pvalue, "code");
+            }
+            if (code == NULL) {
+                /*
+                 * It should be impossible to get here unless the code attribute
+                 * is missing. There is no reasonable way to handle that case so
+                 * clear the error and let the SystemExit become a JepException
+                 */
+                PyErr_Clear();
+            } else if (code == Py_None) {
+                exit(0);
+            } else if (PyLong_Check(code)) {
+                exit(PyLong_AsLong(code));
+            } else {
+                /*
+                 * It should be impossible to get here but if something changes
+                 * it seems reasonable to just let the SystemExit become a JepException
+                 */
+                Py_XDECREF(code);
+            }
+        }
+
         message = PyObject_Str(ptype);
 
         if (pvalue) {
@@ -94,13 +112,6 @@ int process_py_exception(JNIEnv *env)
                  * there is an except: block even if the error is not actually
                  * caught.
                  */
-#if PY_MAJOR_VERSION < 3
-                PyObject *message = PyObject_GetAttrString(pvalue, "message");
-                if (message != NULL && PyJObject_Check(message)) {
-                    Py_DECREF(pvalue);
-                    pvalue = message;
-                }
-#else
                 PyObject *args = PyObject_GetAttrString(pvalue, "args");
                 if (args != NULL && PyTuple_Check(args) && PyTuple_Size(args) > 0) {
                     PyObject *message = PyTuple_GetItem(args, 0);
@@ -109,7 +120,6 @@ int process_py_exception(JNIEnv *env)
                     Py_DECREF(args);
                     pvalue = message;
                 }
-#endif
             }
 
 
@@ -137,19 +147,14 @@ int process_py_exception(JNIEnv *env)
                 v = PyObject_Str(pvalue);
             }
 
-            if (v != NULL && PyString_Check(v)) {
+            if (v != NULL && PyUnicode_Check(v)) {
                 PyObject *t;
-#if PY_MAJOR_VERSION >= 3
                 t = PyUnicode_FromFormat("%U: %U", message, v);
-#else
-                t = PyString_FromFormat("%s: %s", PyString_AsString(message),
-                                        PyString_AsString(v));
-#endif
                 Py_DECREF(v);
                 Py_DECREF(message);
                 message = t;
             }
-            m = PyString_AsString(message);
+            m = PyUnicode_AsUTF8(message);
 
             // make a JepException
             jmsg = (*env)->NewStringUTF(env, (const char *) m);
@@ -173,8 +178,8 @@ int process_py_exception(JNIEnv *env)
             }
             (*env)->DeleteLocalRef(env, jmsg);
             if ((*env)->ExceptionCheck(env) || !jepException) {
-                PyErr_Format(PyExc_RuntimeError,
-                             "creating jep.JepException failed.");
+                PyErr_SetString(PyExc_RuntimeError,
+                                "creating jep.JepException failed.");
                 return 1;
             }
 
@@ -187,7 +192,7 @@ int process_py_exception(JNIEnv *env)
                 if (modTB == NULL) {
                     printf("Error importing python traceback module\n");
                 }
-                extract = PyString_FromString("extract_tb");
+                extract = PyUnicode_FromString("extract_tb");
                 if (extract == NULL) {
                     printf("Error making PyString 'extract_tb'\n");
                 }
@@ -230,8 +235,8 @@ int process_py_exception(JNIEnv *env)
                 stackArray = (*env)->NewObjectArray(env, (jsize) stackSize,
                                                     stackTraceElemClazz, NULL);
                 if ((*env)->ExceptionCheck(env) || !stackArray) {
-                    PyErr_Format(PyExc_RuntimeError,
-                                 "creating java.lang.StackTraceElement[] failed.");
+                    PyErr_SetString(PyExc_RuntimeError,
+                                    "creating java.lang.StackTraceElement[] failed.");
                     Py_DECREF(pystack);
                     return 1;
                 }
@@ -249,11 +254,11 @@ int process_py_exception(JNIEnv *env)
                     stackEntry = PyList_GetItem(pystack, i);
                     // java order is classname, method name, filename, line number
                     // python order is filename, line number, function name, line
-                    charPyFile = PyString_AsString(
+                    charPyFile = PyUnicode_AsUTF8(
                                      PySequence_GetItem(stackEntry, 0));
-                    pyLineNum = (int) PyInt_AsLong(
+                    pyLineNum = (int) PyLong_AsLongLong(
                                     PySequence_GetItem(stackEntry, 1));
-                    charPyFunc = PyString_AsString(
+                    charPyFunc = PyUnicode_AsUTF8(
                                      PySequence_GetItem(stackEntry, 2));
                     pyLine = PySequence_GetItem(stackEntry, 3);
 
@@ -279,7 +284,7 @@ int process_py_exception(JNIEnv *env)
                         }
 
                         // remove the dir path to look more like a Java StackTraceElement
-                        charPyFileNoDir = malloc(sizeof(char) * (namelen + 1));
+                        charPyFileNoDir = malloc(namelen + 1);
                         lastBackslash = strrchr(charPyFile, FILE_SEP);
                         if (lastBackslash != NULL) {
                             strcpy(charPyFileNoDir, lastBackslash + 1);
@@ -329,8 +334,8 @@ int process_py_exception(JNIEnv *env)
                  */
                 javaStack = java_lang_Throwable_getStackTrace(env, jepException);
                 if ((*env)->ExceptionCheck(env) || !javaStack) {
-                    PyErr_Format(PyExc_RuntimeError,
-                                 "Lookup of current java stack failed.");
+                    PyErr_SetString(PyExc_RuntimeError,
+                                    "Lookup of current java stack failed.");
                     return 1;
                 }
                 javaStackLength = (*env)->GetArrayLength(env, javaStack);
@@ -343,8 +348,8 @@ int process_py_exception(JNIEnv *env)
                 reverse = (*env)->NewObjectArray(env, (jsize) (count + javaStackLength),
                                                  stackTraceElemClazz, NULL);
                 if ((*env)->ExceptionCheck(env) || !reverse) {
-                    PyErr_Format(PyExc_RuntimeError,
-                                 "creating reverse java.lang.StackTraceElement[] failed.");
+                    PyErr_SetString(PyExc_RuntimeError,
+                                    "creating reverse java.lang.StackTraceElement[] failed.");
                     return 1;
                 }
 
@@ -390,9 +395,9 @@ int process_py_exception(JNIEnv *env)
     if (jepException != NULL) {
         Py_DECREF(message);
         THROW_JEP_EXC(env, jepException);
-    } else if (message && PyString_Check(message)) {
+    } else if (message && PyUnicode_Check(message)) {
         // should only get here if there was a ptype but no pvalue
-        m = PyString_AsString(message);
+        m = PyUnicode_AsUTF8(message);
         THROW_JEP(env, m);
         Py_DECREF(message);
     }
@@ -400,52 +405,6 @@ int process_py_exception(JNIEnv *env)
     return 1;
 }
 
-
-// convert java exception to ImportError.
-// true (1) if an exception was processed.
-int process_import_exception(JNIEnv *env)
-{
-    PyObject   *estr;
-    jthrowable  exception    = NULL;
-    PyObject   *pyException  = PyExc_ImportError;
-    JepThread  *jepThread;
-
-    if (!(*env)->ExceptionCheck(env)) {
-        return 0;
-    }
-
-    if ((exception = (*env)->ExceptionOccurred(env)) == NULL) {
-        return 0;
-    }
-
-    jepThread = pyembed_get_jepthread();
-    if (!jepThread) {
-        printf("Error while processing a Java exception, "
-               "invalid JepThread.\n");
-        return 1;
-    }
-
-    // we're already processing this one, clear the old
-    (*env)->ExceptionClear(env);
-
-    if ((*env)->ExceptionCheck(env)) {
-        (*env)->ExceptionDescribe(env);
-        return 1;
-    }
-
-    estr = jobject_As_PyString(env, exception);
-    if (!estr) {
-        return 1;
-    }
-
-    PyErr_SetObject(pyException, estr);
-    Py_DECREF(estr);
-
-    (*env)->DeleteLocalRef(env, exception);
-    return 1;
-}
-
-
 /*
  * Converts a Java exception to a PyErr.  Returns true if an exception was
  * processed.  If an error was processed here, it can be caught in Python code
@@ -456,7 +415,6 @@ int process_java_exception(JNIEnv *env)
     jthrowable exception = NULL;
     PyObject *pyExceptionType;
     PyObject *jpyExc;
-    JepThread *jepThread;
     jobjectArray stack;
 
     if (!(*env)->ExceptionCheck(env)) {
@@ -477,13 +435,6 @@ int process_java_exception(JNIEnv *env)
         PyErr_Print();
     }
 
-    jepThread = pyembed_get_jepthread();
-    if (!jepThread) {
-        printf("Error while processing a Java exception, "
-               "invalid JepThread.\n");
-        return 1;
-    }
-
     // we're already processing this one, clear the old
     (*env)->ExceptionClear(env);
 
@@ -494,22 +445,21 @@ int process_java_exception(JNIEnv *env)
      */
     stack = java_lang_Throwable_getStackTrace(env, exception);
     if ((*env)->ExceptionCheck(env)) {
-        PyErr_Format(PyExc_RuntimeError,
-                     "wrapping java exception in pyjobject failed.");
+        PyErr_SetString(PyExc_RuntimeError,
+                        "wrapping java exception in pyjobject failed.");
         return 1;
 
     }
-    (*env)->DeleteLocalRef(env, stack);
     pyExceptionType = pyerrtype_from_throwable(env, exception);
 
-    // turn the Java exception into a PyJObject so the interpreter can handle it
-    jpyExc = PyJObject_Wrap(env, exception, NULL);
+    jpyExc = jobject_As_PyObject(env, exception);
     if (!jpyExc) {
         return 1;
     }
 
     PyErr_SetObject(pyExceptionType, jpyExc);
     Py_DECREF(jpyExc);
+    (*env)->DeleteLocalRef(env, stack);
     (*env)->DeleteLocalRef(env, exception);
     return 1;
 }
