@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2015-2019 JEP AUTHORS.
+   Copyright (c) 2015-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -55,22 +55,13 @@ static jobject NATIVE_BYTE_ORDER  = NULL;
 
 /*
  * Initializes the numpy extension library.  This is required to be called
- * once and only once, before any PyArray_ methods are called. Unfortunately
- * it needs to have a different return type in Python 2 vs Python 3.
+ * once and only once, before any PyArray_ methods are called.
  */
-#if PY_MAJOR_VERSION >= 3
 static PyObject* _init_numpy(void)
 {
     import_array();
     return NULL;
 }
-#else
-static void _init_numpy(void)
-{
-    import_array();
-}
-#endif // Python 3 compatibility
-
 static int init_numpy(void)
 {
     if (!numpyInitialized) {
@@ -402,7 +393,6 @@ static jobject convert_pyndarray_jndarray(JNIEnv *env, PyObject *pyobj)
     for (i = 0; i < ndims; i++) {
         jdims[i] = (jint) dims[i];
     }
-
     jdimObj = (*env)->NewIntArray(env, ndims);
     if (process_java_exception(env) || !jdimObj) {
         free(jdims);
@@ -598,6 +588,7 @@ PyObject* convert_jdndarray_pyndarray(JNIEnv *env, PyObject* pyobj)
     // get the primitive array and convert it
     data = (*env)->CallObjectMethod(env, obj, dndarrayGetData);
     if (process_java_exception(env) || !data) {
+        free(dims);
         return NULL;
     }
 
@@ -747,6 +738,7 @@ PyObject* convert_jndarray_pyndarray(JNIEnv *env, jobject obj)
     // get the primitive array and convert it
     data = (*env)->CallObjectMethod(env, obj, ndarrayGetData);
     if (process_java_exception(env) || !data) {
+        free(dims);
         return NULL;
     }
 
