@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2004-2019 JEP AUTHORS.
+   Copyright (c) 2004-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -427,54 +427,12 @@ int pyarg_matches_jtype(JNIEnv *env,
                 return 2;
             }
         }
-    } else if (PyInt_Check(param)) {
-        switch (paramTypeId) {
-        case JINT_ID:
-            return 11;
-        case JLONG_ID:
-            return 10;
-        case JDOUBLE_ID:
-            return 9;
-        case JFLOAT_ID:
-            return 8;
-        case JSHORT_ID:
-            return 7;
-        case JBYTE_ID:
-            return 6;
-        case JBOOLEAN_ID:
-            return 5;
-        case JOBJECT_ID:
-            if ((*env)->IsSameObject(env, JINT_OBJ_TYPE, paramType)) {
-                return 4;
-            } else if ((*env)->IsSameObject(env, JLONG_OBJ_TYPE, paramType)) {
-                return 3;
-            } else if ((*env)->IsSameObject(env, JOBJECT_TYPE, paramType)) {
-                return 1;
-            } else if ((*env)->IsAssignableFrom(env, JINT_OBJ_TYPE, paramType)) {
-                return 2;
-            }
-        }
-    } else if (PyString_Check(param)) {
-        switch (paramTypeId) {
-        case JSTRING_ID:
-            return 3;
-        case JCHAR_ID:
-            if (PyString_GET_SIZE(param) == 1) {
-                return 2;
-            }
-            break;
-        case JOBJECT_ID:
-            if ((*env)->IsAssignableFrom(env, JSTRING_TYPE, paramType)) {
-                return 1;
-            }
-        }
     } else if (PyUnicode_Check(param)) {
         switch (paramTypeId) {
         case JSTRING_ID:
             return 3;
-            break;
         case JCHAR_ID:
-            if (PyUnicode_GET_SIZE(param) == 1) {
+            if (PyUnicode_GET_LENGTH(param) == 1) {
                 return 2;
             }
             break;
@@ -595,7 +553,7 @@ jvalue convert_pyarg_jvalue(JNIEnv *env, PyObject *param, jclass paramType,
             pvalue_string = PyObject_Str(pvalue);
         }
         PyErr_Format(PyExc_TypeError, "Error converting parameter %d: %s", pos + 1,
-                     PyString_AsString(pvalue_string));
+                     PyUnicode_AsUTF8(pvalue_string));
         Py_DECREF(pvalue_string);
         Py_DECREF(ptype);
         Py_XDECREF(pvalue);
