@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2004-2019 JEP AUTHORS.
+   Copyright (c) 2004-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -100,13 +100,27 @@ static struct PyMethodDef jep_methods[] = {
         "jarray",
         pyjarray_new_v,
         METH_VARARGS,
-        "Create a new primitive array in Java.\n"
-        "Accepts:\n"
-        "(size, type _ID, [0]) || "
-        "(size, JCHAR_ID, [string value] || "
-        "(size, jobject) || "
-        "(size, str) || "
-        "(size, jarray)"
+        "Returns a new Java Array.\n"
+        "Accepts:\n  "
+        "(size, typecode, [fill]) or "
+        "(size, jclass)\n"
+        "\n"
+        "Java arrays behave much like lists, except the size is fixed at\n"
+        "creation and all the components of a jarray have the same type.\n"
+        "The component type is specified at object creation time by using a\n"
+        "jclass or type code. Java Object arrays are created by using a jclass\n"
+        "for the component type and Java primitive arrays are created by using\n"
+        "a type code for the component type.\n"
+        "The following type codes are defined:\n"
+        "  Type code   Java Primitive Type\n"
+        "  'z'         boolean\n"
+        "  'b'         byte\n"
+        "  'c'         char\n"
+        "  's'         short\n"
+        "  'i'         int\n"
+        "  'j'         long\n"
+        "  'f'         float\n"
+        "  'd'         double\n"
     },
 
     {
@@ -120,8 +134,6 @@ static struct PyMethodDef jep_methods[] = {
 
     { NULL, NULL }
 };
-
-#if PY_MAJOR_VERSION >= 3
 static struct PyModuleDef jep_module_def = {
     PyModuleDef_HEAD_INIT,
     "_jep",              /* m_name */
@@ -133,60 +145,68 @@ static struct PyModuleDef jep_module_def = {
     NULL,                /* m_clear */
     NULL,                /* m_free */
 };
-#endif
 
-static PyObject* initjep(JNIEnv *env, jboolean hasSharedModules)
+/*
+ * Initialize the _jep module.
+ *
+ * Return 0 on success and -1 on failure.
+ */
+static int initjep(JNIEnv *env, jboolean hasSharedModules)
 {
-    PyObject *modjep;
-
-#if PY_MAJOR_VERSION >= 3
-    PyObject *sysmodules;
-    modjep = PyModule_Create(&jep_module_def);
-    if (modjep == NULL) {
-        handle_startup_exception(env, "Couldn't create module _jep");
-        return NULL;
-    }
-    sysmodules = PyImport_GetModuleDict();
-    if (PyDict_SetItemString(sysmodules, "_jep", modjep) == -1) {
-        handle_startup_exception(env, "Couldn't set _jep on sys.modules");
-        return NULL;
-    }
-#else
-    modjep = PyImport_AddModule("_jep");
-    if (modjep == NULL) {
-        handle_startup_exception(env, "Couldn't add module _jep");
-        return NULL;
-    }
-    modjep = Py_InitModule((char *) "_jep", jep_methods);
-    if (modjep == NULL) {
-        handle_startup_exception(env, "Couldn't init module _jep");
-        return NULL;
-    }
-#endif
-    modjep = PyImport_ImportModule("_jep");
-    if (modjep == NULL) {
-        handle_startup_exception(env, "Couldn't import module _jep");
-    } else {
-        // stuff for making new pyjarray objects
-        PyModule_AddIntConstant(modjep, "JBOOLEAN_ID", JBOOLEAN_ID);
-        PyModule_AddIntConstant(modjep, "JINT_ID", JINT_ID);
-        PyModule_AddIntConstant(modjep, "JLONG_ID", JLONG_ID);
-        PyModule_AddIntConstant(modjep, "JSTRING_ID", JSTRING_ID);
-        PyModule_AddIntConstant(modjep, "JDOUBLE_ID", JDOUBLE_ID);
-        PyModule_AddIntConstant(modjep, "JSHORT_ID", JSHORT_ID);
-        PyModule_AddIntConstant(modjep, "JFLOAT_ID", JFLOAT_ID);
-        PyModule_AddIntConstant(modjep, "JCHAR_ID", JCHAR_ID);
-        PyModule_AddIntConstant(modjep, "JBYTE_ID", JBYTE_ID);
+    PyObject *sysmodules = PyImport_GetModuleDict();
+    PyObject *modjep = PyDict_GetItemString(sysmodules, "_jep");
+    if (!modjep && PyErr_Occurred()) {
+        handle_startup_exception(env, "Error checking for exisitng module _jep");
+    } else if (!modjep) {
+        modjep = PyModule_Create(&jep_module_def);
+        if (!modjep) {
+            handle_startup_exception(env, "Couldn't create module _jep");
+            return -1;
+        }
+        if (PyDict_SetItemString(sysmodules, "_jep", modjep) == -1) {
+            Py_DECREF(modjep);
+            handle_startup_exception(env, "Couldn't set _jep on sys.modules");
+            return -1;
+        }
+        PyModule_AddStringConstant(modjep, "JBOOLEAN_ID", "z");
+        PyModule_AddStringConstant(modjep, "JINT_ID", "i");
+        PyModule_AddStringConstant(modjep, "JLONG_ID", "j");
+        PyModule_AddStringConstant(modjep, "JDOUBLE_ID", "d");
+        PyModule_AddStringConstant(modjep, "JSHORT_ID", "s");
+        PyModule_AddStringConstant(modjep, "JFLOAT_ID", "f");
+        PyModule_AddStringConstant(modjep, "JCHAR_ID", "c");
+        PyModule_AddStringConstant(modjep, "JBYTE_ID", "b");
         PyModule_AddIntConstant(modjep, "JEP_NUMPY_ENABLED", JEP_NUMPY_ENABLED);
+        PyObject *javaAttrCache = PyDict_New();
+        if (!javaAttrCache) {
+            Py_DECREF(modjep);
+            return -1;
+        }
+        if (PyModule_AddObject(modjep, "__javaAttributeCache__", javaAttrCache)) {
+            Py_DECREF(javaAttrCache);
+            Py_DECREF(modjep);
+            return -1;
+        }
+        PyObject *javaTypeCache = PyDict_New();
+        if (!javaTypeCache) {
+            Py_DECREF(modjep);
+            return -1;
+        }
+        if (PyModule_AddObject(modjep, "__javaTypeCache__", javaTypeCache)) {
+            Py_DECREF(javaTypeCache);
+            Py_DECREF(modjep);
+            return -1;
+        }
         if (hasSharedModules) {
             Py_INCREF(mainThreadModules);
             PyModule_AddObject(modjep, "mainInterpreterModules", mainThreadModules);
             Py_INCREF(mainThreadModulesLock);
             PyModule_AddObject(modjep, "mainInterpreterModulesLock", mainThreadModulesLock);
         }
+        /* still held by sys.modules. */
+        Py_DECREF(modjep);
     }
-
-    return modjep;
+    return 0;
 }
 
 void pyembed_preinit(JNIEnv *env,
@@ -222,21 +242,8 @@ void pyembed_preinit(JNIEnv *env,
     }
     if (pythonHome) {
         const char* homeAsUTF = (*env)->GetStringUTFChars(env, pythonHome, NULL);
-#if PY_MAJOR_VERSION >= 3
-#if PY_MAJOR_VERSION > 3 || PY_MINOR_VERSION >= 5
         wchar_t* homeForPython = Py_DecodeLocale(homeAsUTF, NULL);
-#else
-        int length = (*env)->GetStringUTFLength(env, pythonHome);
-        wchar_t* homeForPython = malloc((length + 1) * sizeof(wchar_t));
-        mbstowcs(homeForPython, homeAsUTF, length + 1);
-#endif
-#else
-        int length = (*env)->GetStringUTFLength(env, pythonHome);
-        char* homeForPython = malloc(length);
-        strncpy(homeForPython, homeAsUTF, length);
-#endif
         (*env)->ReleaseStringUTFChars(env, pythonHome, homeAsUTF);
-
         Py_SetPythonHome(homeForPython);
         // Python documentation says that the string should not be changed for
         // the duration of the program so it can never be freed.
@@ -252,15 +259,16 @@ void pyembed_preinit(JNIEnv *env,
  *
  * Furthermore, we need to ensure that the inheritance tree is built in the
  * correct order, i.e. from the top down.  For example, we need to set that
- * PyJIterable's tp_base extends PyJObject before we set that PyJCollection's
- * tp_base extends PyJIterable.
+ * PyJCollection's tp_base extends PyJIterable before we set that PyJList's
+ * tp_base extends PyJCollection. Interfaces that are not extending another
+ * interface should not set tp_base because interfaces are added to Python
+ * types using multiple inheritance and only one superclass can define a
+ * custom structure.
  *
- * See https://docs.python.org/2/extending/newtypes.html
- *     https://docs.python.org/3/extending/newtypes.html
+ * See https://docs.python.org/3/extending/newtypes.html
  */
 static int pyjtypes_ready(void)
 {
-    // start at the top with object
     if (PyType_Ready(&PyJObject_Type) < 0) {
         return -1;
     }
@@ -271,7 +279,6 @@ static int pyjtypes_ready(void)
         return -1;
     }
 
-    // next do number
     if (!PyJNumber_Type.tp_base) {
         PyJNumber_Type.tp_base = &PyJObject_Type;
     }
@@ -279,23 +286,14 @@ static int pyjtypes_ready(void)
         return -1;
     }
 
-    // next do iterable
-    if (!PyJIterable_Type.tp_base) {
-        PyJIterable_Type.tp_base = &PyJObject_Type;
-    }
     if (PyType_Ready(&PyJIterable_Type) < 0) {
         return -1;
     }
 
-    // next do iterator
-    if (!PyJIterator_Type.tp_base) {
-        PyJIterator_Type.tp_base = &PyJObject_Type;
-    }
     if (PyType_Ready(&PyJIterator_Type) < 0) {
         return -1;
     }
 
-    // next do collection
     if (!PyJCollection_Type.tp_base) {
         PyJCollection_Type.tp_base = &PyJIterable_Type;
     }
@@ -303,7 +301,6 @@ static int pyjtypes_ready(void)
         return -1;
     }
 
-    // next do list
     if (!PyJList_Type.tp_base) {
         PyJList_Type.tp_base = &PyJCollection_Type;
     }
@@ -311,10 +308,6 @@ static int pyjtypes_ready(void)
         return -1;
     }
 
-    // next do map
-    if (!PyJMap_Type.tp_base) {
-        PyJMap_Type.tp_base = &PyJObject_Type;
-    }
     if (PyType_Ready(&PyJMap_Type) < 0) {
         return -1;
     }
@@ -326,10 +319,6 @@ static int pyjtypes_ready(void)
         return -1;
     }
 
-    // last do autocloseable
-    if (!PyJAutoCloseable_Type.tp_base) {
-        PyJAutoCloseable_Type.tp_base = &PyJObject_Type;
-    }
     if (PyType_Ready(&PyJAutoCloseable_Type) < 0) {
         return -1;
     }
@@ -406,7 +395,10 @@ void pyembed_startup(JNIEnv *env, jobjectArray sharedModulesArgv)
     }
 
     Py_Initialize();
+
+#if PY_MAJOR_VERSION < 4 && PY_MINOR_VERSION < 7
     PyEval_InitThreads();
+#endif
 
     if (pyjtypes_ready()) {
         handle_startup_exception(env, "Failed to initialize PyJTypes");
@@ -457,39 +449,6 @@ void pyembed_startup(JNIEnv *env, jobjectArray sharedModulesArgv)
      * See github issue #81.
      */
     if (sharedModulesArgv != NULL) {
-#if PY_MAJOR_VERSION < 3
-        char **argv = NULL;
-        jsize count = 0;
-        int i       = 0;
-
-        count = (*env)->GetArrayLength(env, sharedModulesArgv);
-        (*env)->PushLocalFrame(env, count * 2);
-        argv = (char**) malloc(count * sizeof(char*));
-        for (i = 0; i < count; i++) {
-            char* arg = NULL;
-
-            jstring jarg = (*env)->GetObjectArrayElement(env, sharedModulesArgv, i);
-            if (jarg == NULL) {
-                PyEval_ReleaseThread(mainThreadState);
-                (*env)->PopLocalFrame(env, NULL);
-                THROW_JEP(env, "Received null argv.");
-                return;
-            }
-            arg = (char*) (*env)->GetStringUTFChars(env, jarg, NULL);
-            argv[i] = arg;
-        }
-
-        PySys_SetArgvEx(count, argv, 0);
-
-        // free memory
-        for (i = 0; i < count; i++) {
-            jstring jarg = (*env)->GetObjectArrayElement(env, sharedModulesArgv, i);
-            (*env)->ReleaseStringUTFChars(env, jarg, argv[i]);
-        }
-        free(argv);
-        (*env)->PopLocalFrame(env, NULL);
-
-#else
         wchar_t **argv = NULL;
         jsize count = 0;
         int i       = 0;
@@ -500,12 +459,16 @@ void pyembed_startup(JNIEnv *env, jobjectArray sharedModulesArgv)
         for (i = 0; i < count; i++) {
             char* arg     = NULL;
             wchar_t* argt = NULL;
-
+            int k = 0;
             jstring jarg = (*env)->GetObjectArrayElement(env, sharedModulesArgv, i);
             if (jarg == NULL) {
                 PyEval_ReleaseThread(mainThreadState);
                 (*env)->PopLocalFrame(env, NULL);
                 THROW_JEP(env, "Received null argv.");
+                for (k = 0; k < i; k++) {
+                    free(argv[k]);
+                }
+                free(argv);
                 return;
             }
             arg = (char*) (*env)->GetStringUTFChars(env, jarg, NULL);
@@ -523,8 +486,6 @@ void pyembed_startup(JNIEnv *env, jobjectArray sharedModulesArgv)
         }
         free(argv);
         (*env)->PopLocalFrame(env, NULL);
-
-#endif
     }
 
     PyEval_ReleaseThread(mainThreadState);
@@ -545,7 +506,7 @@ int pyembed_is_version_unsafe(void)
     int         i         = 0;
 
     pyversion = Py_GetVersion();
-    version = malloc(sizeof(char) * (strlen(pyversion) + 1));
+    version = malloc(strlen(pyversion) + 1);
     strcpy(version, pyversion);
     major = version;
 
@@ -620,79 +581,6 @@ void pyembed_shared_import(JNIEnv *env, jstring module)
     PyEval_ReleaseThread(mainThreadState);
 }
 
-#if PY_MAJOR_VERSION < 3
-/*
- * In python 2.7 when a module is shared between threads it will be run in
- * restricted mode because the __builtins__ attribute of the module does not
- * match the running interpreter. This causes some functionality to break.
- * To get around this limitation the builtins module is shared between all the
- * interpreters that are using shared modules.
- *
- * This is not necessary in python 3 because restricted mode was removed.
- */
-
-void assignBuiltins(PyObject* modules, PyObject* builtins)
-{
-    PyObject *key, *value;
-    Py_ssize_t pos = 0;
-
-    while (PyDict_Next(modules, &pos, &key, &value)) {
-        PyObject* moddict = PyModule_GetDict(value);
-        if (moddict) {
-            PyDict_SetItemString(moddict, "__builtins__", builtins);
-        }
-    }
-}
-
-void shareBuiltins(JepThread *jepThread)
-{
-    PyInterpreterState* interp = jepThread->tstate->interp;
-
-    PyObject* sharedBuiltinModule = PyDict_GetItemString(mainThreadModules,
-                                    "__builtin__");
-    PyObject* sharedBuiltinDict   = PyModule_GetDict(sharedBuiltinModule);
-
-    PyObject* originalBuiltinModule = PyDict_GetItemString(interp->modules,
-                                      "__builtin__");
-    PyObject* originalBuiltinDict   = interp->builtins;
-
-    Py_INCREF(sharedBuiltinDict);
-    interp->builtins = sharedBuiltinDict;
-    Py_DECREF(originalBuiltinDict);
-
-    // Incref to make sure this isn't collected when it is replaced, it
-    // will be restored when the itnterpreter is closed.
-    Py_INCREF(originalBuiltinModule);
-
-    PyDict_SetItemString(interp->modules, "__builtin__", sharedBuiltinModule);
-    assignBuiltins(interp->modules, sharedBuiltinDict);
-
-    jepThread->originalBuiltins = originalBuiltinModule;
-}
-
-void unshareBuiltins(JepThread *jepThread)
-{
-    PyInterpreterState* interp = jepThread->tstate->interp;
-
-    PyObject* originalBuiltinModule = jepThread->originalBuiltins;
-    PyObject* originalBuiltinDict   = PyModule_GetDict(originalBuiltinModule);
-
-    PyObject* sharedBuiltinDict   = interp->builtins;
-
-    Py_INCREF(originalBuiltinDict);
-    interp->builtins = originalBuiltinDict;
-    Py_DECREF(sharedBuiltinDict);
-
-    PyDict_SetItemString(interp->modules, "__builtin__", originalBuiltinModule);
-
-    assignBuiltins(interp->modules, originalBuiltinDict);
-
-    jepThread->originalBuiltins = NULL;
-    Py_DECREF(originalBuiltinModule);
-}
-
-#endif
-
 intptr_t pyembed_thread_init(JNIEnv *env, jobject cl, jobject caller,
                              jboolean hasSharedModules, jboolean usesubinterpreter)
 {
@@ -719,13 +607,6 @@ intptr_t pyembed_thread_init(JNIEnv *env, jobject cl, jobject caller,
         PyEval_AcquireThread(mainThreadState);
 
         jepThread->tstate = Py_NewInterpreter();
-#if PY_MAJOR_VERSION < 3
-        if (hasSharedModules) {
-            shareBuiltins(jepThread);
-        } else {
-            jepThread->originalBuiltins = NULL;
-        }
-#endif
 
         /*
          * Py_NewInterpreter() seems to take the thread state, but we're going to
@@ -734,9 +615,6 @@ intptr_t pyembed_thread_init(JNIEnv *env, jobject cl, jobject caller,
         PyEval_SaveThread();
     } else {
         jepThread->tstate = PyThreadState_New(mainThreadState->interp);
-#if PY_MAJOR_VERSION < 3
-        jepThread->originalBuiltins = NULL;
-#endif
     }
     PyEval_AcquireThread(jepThread->tstate);
 
@@ -754,6 +632,7 @@ intptr_t pyembed_thread_init(JNIEnv *env, jobject cl, jobject caller,
         if (mod_main == NULL) {
             THROW_JEP(env, "Couldn't add module __main__.");
             PyEval_ReleaseThread(jepThread->tstate);
+            free(jepThread);
             return 0;
         }
         globals = PyModule_GetDict(mod_main);
@@ -762,24 +641,20 @@ intptr_t pyembed_thread_init(JNIEnv *env, jobject cl, jobject caller,
         globals = PyDict_New();
         PyDict_SetItemString(globals, "__builtins__", PyEval_GetBuiltins());
     }
+    if (initjep(env, hasSharedModules)) {
+        return 0;
+    }
 
     // init static module
-    jepThread->modjep          = initjep(env, hasSharedModules);
     jepThread->globals         = globals;
     jepThread->env             = env;
     jepThread->classloader     = (*env)->NewGlobalRef(env, cl);
     jepThread->caller          = (*env)->NewGlobalRef(env, caller);
-    jepThread->fqnToPyJAttrs = NULL;
 
     if ((tdict = PyThreadState_GetDict()) != NULL) {
         PyObject *key, *t;
-
-#if PY_MAJOR_VERSION >= 3
         t   = PyCapsule_New((void *) jepThread, NULL, NULL);
-#else
-        t   = (PyObject *) PyCObject_FromVoidPtr((void *) jepThread, NULL);
-#endif
-        key = PyString_FromString(DICT_KEY);
+        key = PyUnicode_FromString(DICT_KEY);
 
         PyDict_SetItem(tdict, key, t);   /* takes ownership */
 
@@ -804,20 +679,13 @@ void pyembed_thread_close(JNIEnv *env, intptr_t _jepThread)
     }
 
     PyEval_AcquireThread(jepThread->tstate);
-#if PY_MAJOR_VERSION < 3
-    if (jepThread->originalBuiltins) {
-        unshareBuiltins(jepThread);
-    }
-#endif
-    key = PyString_FromString(DICT_KEY);
+    key = PyUnicode_FromString(DICT_KEY);
     if ((tdict = PyThreadState_GetDict()) != NULL && key != NULL) {
         PyDict_DelItem(tdict, key);
     }
     Py_DECREF(key);
 
     Py_CLEAR(jepThread->globals);
-    Py_CLEAR(jepThread->fqnToPyJAttrs);
-    Py_CLEAR(jepThread->modjep);
 
     if (jepThread->classloader) {
         (*env)->DeleteGlobalRef(env, jepThread->classloader);
@@ -831,7 +699,7 @@ void pyembed_thread_close(JNIEnv *env, intptr_t _jepThread)
         PyThreadState_Delete(jepThread->tstate);
     } else {
         Py_EndInterpreter(jepThread->tstate);
-	PyThreadState_Swap(mainThreadState);
+        PyThreadState_Swap(mainThreadState);
         PyEval_ReleaseThread(mainThreadState);
     }
     free(jepThread);
@@ -867,15 +735,11 @@ JepThread* pyembed_get_jepthread(void)
     PyObject  *tdict, *t, *key;
     JepThread *ret = NULL;
 
-    key = PyString_FromString(DICT_KEY);
+    key = PyUnicode_FromString(DICT_KEY);
     if ((tdict = PyThreadState_GetDict()) != NULL && key != NULL) {
         t = PyDict_GetItem(tdict, key); /* borrowed */
         if (t != NULL && !PyErr_Occurred()) {
-#if PY_MAJOR_VERSION >= 3
             ret = (JepThread*) PyCapsule_GetPointer(t, NULL);
-#else
-            ret = (JepThread*) PyCObject_AsVoidPtr(t);
-#endif
         }
     }
     Py_XDECREF(key);
@@ -932,11 +796,11 @@ static PyObject* pyembed_jproxy(PyObject *self, PyObject *args)
         PyObject   *item;
 
         item = PyList_GET_ITEM(interfaces, i);
-        if (!PyString_Check(item)) {
+        if (!PyUnicode_Check(item)) {
             return PyErr_Format(PyExc_ValueError, "Item %zd not a string.", i);
         }
 
-        str  = PyString_AsString(item);
+        str  = PyUnicode_AsUTF8(item);
         jstr = (*env)->NewStringUTF(env, (const char *) str);
 
         (*env)->SetObjectArrayElement(env, classes, (jsize) i, jstr);
@@ -956,7 +820,9 @@ static PyObject* pyembed_jproxy(PyObject *self, PyObject *args)
     // make sure target doesn't get garbage collected
     Py_INCREF(pytarget);
 
-    result = PyJObject_Wrap(env, proxy, NULL);
+    jclass clazz = (*env)->GetObjectClass(env, proxy);
+    result = jobject_As_PyJObject(env, proxy, clazz);
+    (*env)->DeleteLocalRef(env, clazz);
     (*env)->DeleteLocalRef(env, proxy);
     return result;
 }
@@ -1371,92 +1237,6 @@ void pyembed_exec(JNIEnv *env, intptr_t _jepThread, char *str)
     PyEval_ReleaseThread(jepThread->tstate);
 }
 
-
-void pyembed_setloader(JNIEnv *env, intptr_t _jepThread, jobject cl)
-{
-    jobject    oldLoader = NULL;
-    JepThread *jepThread = (JepThread *) _jepThread;
-    if (!jepThread) {
-        THROW_JEP(env, "Couldn't get thread objects.");
-        return;
-    }
-
-    if (!cl) {
-        return;
-    }
-
-    PyEval_AcquireThread(jepThread->tstate);
-    Py_CLEAR(jepThread->fqnToPyJAttrs);
-
-    oldLoader = jepThread->classloader;
-    if (oldLoader) {
-        (*env)->DeleteGlobalRef(env, oldLoader);
-    }
-
-    jepThread->classloader = (*env)->NewGlobalRef(env, cl);
-    PyEval_ReleaseThread(jepThread->tstate);
-}
-jobject pyembed_getvalue_on(JNIEnv *env,
-                            intptr_t _jepThread,
-                            intptr_t _onModule,
-                            char *str)
-{
-    PyObject       *dict, *result, *onModule;
-    jobject         ret = NULL;
-    JepThread      *jepThread;
-
-    result = 0;
-
-    jepThread = (JepThread *) _jepThread;
-    if (!jepThread) {
-        THROW_JEP(env, "Couldn't get thread objects.");
-        return NULL;
-    }
-
-    if (str == NULL) {
-        return NULL;
-    }
-
-    PyEval_AcquireThread(jepThread->tstate);
-
-    if (process_py_exception(env)) {
-        goto EXIT;
-    }
-
-    onModule = (PyObject *) _onModule;
-    if (!PyModule_Check(onModule)) {
-        THROW_JEP(env, "pyembed_getvalue_on: Invalid onModule.");
-        goto EXIT;
-    }
-
-    dict = PyModule_GetDict(onModule);
-    Py_INCREF(dict);
-
-    result = PyRun_String(str, Py_eval_input, dict, dict);      /* new ref */
-
-    process_py_exception(env);
-    Py_DECREF(dict);
-
-    if (result == NULL) {
-        goto EXIT;    /* don't return, need to release GIL */
-    }
-    if (result == Py_None) {
-        goto EXIT;
-    }
-
-    // convert results to jobject
-    ret = PyObject_As_jobject(env, result, JOBJECT_TYPE);
-    if (!ret) {
-        process_py_exception(env);
-    }
-
-EXIT:
-    Py_XDECREF(result);
-    PyEval_ReleaseThread(jepThread->tstate);
-    return ret;
-}
-
-
 jobject pyembed_getvalue(JNIEnv *env, intptr_t _jepThread, char *str,
                          jclass clazz)
 {
@@ -1505,74 +1285,6 @@ EXIT:
     return ret;
 }
 
-
-
-jobject pyembed_getvalue_array(JNIEnv *env, intptr_t _jepThread, char *str)
-{
-    PyObject       *result;
-    jobject         ret = NULL;
-    JepThread      *jepThread;
-
-    result = NULL;
-
-    jepThread = (JepThread *) _jepThread;
-    if (!jepThread) {
-        THROW_JEP(env, "Couldn't get thread objects.");
-        return NULL;
-    }
-
-    if (str == NULL) {
-        return NULL;
-    }
-
-    PyEval_AcquireThread(jepThread->tstate);
-
-    if (process_py_exception(env)) {
-        goto EXIT;
-    }
-
-    result = PyRun_String(str,  /* new ref */
-                          Py_eval_input,
-                          jepThread->globals,
-                          jepThread->globals);
-
-    process_py_exception(env);
-
-    if (result == NULL || result == Py_None) {
-        goto EXIT;    /* don't return, need to release GIL */
-    }
-
-#if PY_MAJOR_VERSION >= 3
-    if (PyBytes_Check(result) == 0) {
-        PyObject *temp = PyBytes_FromObject(result);
-        if (process_py_exception(env) || result == NULL) {
-            goto EXIT;
-        } else {
-            Py_DECREF(result);
-            result = temp;
-        }
-    }
-#endif
-
-    if (PyBytes_Check(result)) {
-        void *s = (void*) PyBytes_AS_STRING(result);
-        Py_ssize_t n = PyBytes_Size(result);
-        ret = (*env)->NewByteArray(env, (jsize) n);
-        (*env)->SetByteArrayRegion(env, ret, 0, (jsize) n, (jbyte *) s);
-    } else {
-        THROW_JEP(env, "Value is not a string.");
-        goto EXIT;
-    }
-
-
-EXIT:
-    Py_XDECREF(result);
-    PyEval_ReleaseThread(jepThread->tstate);
-    return ret;
-}
-
-
-
 void pyembed_run(JNIEnv *env,
                  intptr_t _jepThread,
                  char *file)
@@ -1634,7 +1346,7 @@ EXIT:
 }
 
 
-// gratuitously copyied from pythonrun.c::run_pyc_file
+// gratuitously copied from pythonrun.c::run_pyc_file
 static void pyembed_run_pyc(JepThread *jepThread,
                             FILE *fp)
 {
@@ -1650,14 +1362,12 @@ static void pyembed_run_pyc(JepThread *jepThread,
         return;
     }
     (void) PyMarshal_ReadLongFromFile(fp);
-#if PY_MAJOR_VERSION >= 3
     // Python 3.3 added an extra long containing the size of the source.
     // https://github.com/python/cpython/commit/5136ac0ca21a05691978df8d0650f902c8ca3463
     (void) PyMarshal_ReadLongFromFile(fp);
 #if PY_MAJOR_VERSION > 3 || PY_MINOR_VERSION >= 7
     // PEP 552 added another long
     (void) PyMarshal_ReadLongFromFile(fp);
-#endif
 #endif
     v = (PyObject *) (intptr_t) PyMarshal_ReadLastObjectFromFile(fp);
     if (v == NULL || !PyCode_Check(v)) {
@@ -1666,19 +1376,14 @@ static void pyembed_run_pyc(JepThread *jepThread,
         return;
     }
     co = v;
-#if PY_MAJOR_VERSION >= 3
     v = PyEval_EvalCode(co, jepThread->globals, jepThread->globals);
-#else
-    v = PyEval_EvalCode((PyCodeObject *) co, jepThread->globals,
-                        jepThread->globals);
-#endif
     Py_DECREF(co);
     Py_XDECREF(v);
 }
 
 /* Check whether a file maybe a pyc file: Look at the extension,
  the file type, and, if we may close it, at the first few bytes. */
-// gratuitously copyied from pythonrun.c::run_pyc_file
+// gratuitously copied from pythonrun.c::run_pyc_file
 static int maybe_pyc_file(FILE *fp, const char* filename, const char* ext,
                           int closeit)
 {
@@ -1774,282 +1479,4 @@ void pyembed_setparameter_object(JNIEnv *env,
     process_py_exception(env);
 
     PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-
-void pyembed_setparameter_array(JNIEnv *env,
-                                intptr_t _jepThread,
-                                intptr_t module,
-                                const char *name,
-                                jobjectArray obj)
-{
-    PyObject      *pyjob;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    if (obj == NULL) {
-        Py_INCREF(Py_None);
-        pyjob = Py_None;
-    } else {
-        pyjob = pyjarray_new(env, obj);
-    }
-
-    if (pyjob) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyjob); /* ownership */
-            Py_DECREF(pyjob);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyjob); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-
-void pyembed_setparameter_class(JNIEnv *env,
-                                intptr_t _jepThread,
-                                intptr_t module,
-                                const char *name,
-                                jclass value)
-{
-    PyObject      *pyjob;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    if (value == NULL) {
-        Py_INCREF(Py_None);
-        pyjob = Py_None;
-    } else {
-        pyjob = PyJClass_Wrap(env, value);
-    }
-
-    if (pyjob) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyjob); /* ownership */
-            Py_DECREF(pyjob);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyjob); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-
-void pyembed_setparameter_string(JNIEnv *env,
-                                 intptr_t _jepThread,
-                                 intptr_t module,
-                                 const char *name,
-                                 const char *value)
-{
-    PyObject      *pyvalue;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    if (value == NULL) {
-        Py_INCREF(Py_None);
-        pyvalue = Py_None;
-    } else {
-        pyvalue = PyString_FromString(value);
-    }
-
-    if (pyvalue) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyvalue); /* ownership */
-            Py_DECREF(pyvalue);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyvalue); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-void pyembed_setparameter_bool(JNIEnv *env,
-                               intptr_t _jepThread,
-                               intptr_t module,
-                               const char *name,
-                               jboolean value)
-{
-    PyObject      *pyvalue;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    pyvalue = jboolean_As_PyObject(value);
-
-    if (pyvalue) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyvalue); /* ownership */
-            Py_DECREF(pyvalue);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyvalue); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-void pyembed_setparameter_int(JNIEnv *env,
-                              intptr_t _jepThread,
-                              intptr_t module,
-                              const char *name,
-                              int value)
-{
-    PyObject      *pyvalue;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    pyvalue = jint_As_PyObject(value);
-
-    if (pyvalue) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyvalue); /* ownership */
-            Py_DECREF(pyvalue);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyvalue); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-
-void pyembed_setparameter_long(JNIEnv *env,
-                               intptr_t _jepThread,
-                               intptr_t module,
-                               const char *name,
-                               PY_LONG_LONG value)
-{
-    PyObject      *pyvalue;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    pyvalue = jlong_As_PyObject(value);
-
-    if (pyvalue) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyvalue); /* ownership */
-            Py_DECREF(pyvalue);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyvalue); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-
-void pyembed_setparameter_double(JNIEnv *env,
-                                 intptr_t _jepThread,
-                                 intptr_t module,
-                                 const char *name,
-                                 double value)
-{
-    PyObject      *pyvalue;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    pyvalue = jdouble_As_PyObject(value);
-
-    if (pyvalue) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyvalue); /* ownership */
-            Py_DECREF(pyvalue);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyvalue); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
-}
-
-
-void pyembed_setparameter_float(JNIEnv *env,
-                                intptr_t _jepThread,
-                                intptr_t module,
-                                const char *name,
-                                float value)
-{
-    PyObject      *pyvalue;
-    PyObject      *pymodule;
-
-    // does common things
-    GET_COMMON;
-
-    pyvalue = jfloat_As_PyObject((double) value);
-
-    if (pyvalue) {
-        if (pymodule == NULL) {
-            PyDict_SetItemString(jepThread->globals,
-                                 name,
-                                 pyvalue); /* ownership */
-            Py_DECREF(pyvalue);
-        } else {
-            PyModule_AddObject(pymodule,
-                               (char *) name,
-                               pyvalue); // steals reference
-        }
-    }
-    process_py_exception(env);
-
-    PyEval_ReleaseThread(jepThread->tstate);
-    return;
 }
