@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2004-2019 JEP AUTHORS.
+   Copyright (c) 2004-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -102,11 +102,11 @@ PyObject* pyjarray_new_v(PyObject *isnull, PyObject *args)
         return NULL;
     }
 
-    if (PyInt_Check(one)) {
-        size = (long) PyInt_AsLong(one);
+    if (PyLong_Check(one)) {
+        size = (long) PyLong_AsLongLong(one);
 
-        if (PyInt_Check(two)) {
-            typeId = (int) PyInt_AsLong(two);
+        if (PyLong_Check(two)) {
+            typeId = (int) PyLong_AsLongLong(two);
 
             if (size < 0) {
                 return PyErr_Format(PyExc_ValueError, "Invalid size %li", size);
@@ -157,13 +157,92 @@ PyObject* pyjarray_new_v(PyObject *isnull, PyObject *args)
         } // if int(two)
         else if (PyJObject_Check(two)) {
             PyJObject *pyjob = (PyJObject *) two;
-            typeId = JOBJECT_ID;
 
             componentClass = pyjob->clazz;
-            arrayObj = (*env)->NewObjectArray(env,
-                                              (jsize) size,
-                                              componentClass,
-                                              NULL);
+            if ((*env)->IsAssignableFrom(env, componentClass, JOBJECT_TYPE)) {
+                typeId = JOBJECT_ID;
+                arrayObj = (*env)->NewObjectArray(env,
+                                                  (jsize) size,
+                                                  componentClass,
+                                                  NULL);
+            } else if ((*env)->IsSameObject(env, componentClass, JBOOLEAN_TYPE)) {
+                typeId = JBOOLEAN_ID;
+                arrayObj = (*env)->NewBooleanArray(env, (jsize) size);
+            } else if ((*env)->IsSameObject(env, componentClass, JBYTE_TYPE)) {
+                typeId = JBYTE_ID;
+                arrayObj = (*env)->NewByteArray(env, (jsize) size);
+            } else if ((*env)->IsSameObject(env, componentClass, JCHAR_TYPE)) {
+                typeId = JCHAR_ID;
+                arrayObj = (*env)->NewCharArray(env, (jsize) size);
+            } else if ((*env)->IsSameObject(env, componentClass, JSHORT_TYPE)) {
+                typeId = JSHORT_ID;
+                arrayObj = (*env)->NewShortArray(env, (jsize) size);
+            } else if ((*env)->IsSameObject(env, componentClass, JINT_TYPE)) {
+                typeId = JINT_ID;
+                arrayObj = (*env)->NewIntArray(env, (jsize) size);
+            } else if ((*env)->IsSameObject(env, componentClass, JLONG_TYPE)) {
+                typeId = JLONG_ID;
+                arrayObj = (*env)->NewLongArray(env, (jsize) size);
+            } else if ((*env)->IsSameObject(env, componentClass, JFLOAT_TYPE)) {
+                typeId = JFLOAT_ID;
+                arrayObj = (*env)->NewFloatArray(env, (jsize) size);
+            } else if ((*env)->IsSameObject(env, componentClass, JDOUBLE_TYPE)) {
+                typeId = JDOUBLE_ID;
+                arrayObj = (*env)->NewDoubleArray(env, (jsize) size);
+            } else {
+                /* Void? */
+                PyErr_SetString(PyExc_ValueError, "Unsupported jarray Component type");
+                return NULL;
+            }
+        } else if (PyUnicode_Check(two)) {
+            Py_UCS1 typecode = 0;
+            if (PyUnicode_READY(two) != 0) {
+                return NULL;
+            } else if (PyUnicode_GET_LENGTH(two) == 1) {
+                if (PyUnicode_KIND((two)) == PyUnicode_1BYTE_KIND) {
+                    typecode = PyUnicode_1BYTE_DATA(two)[0];
+                }
+            }
+            /* 0 will fall through like any other invalid character. */
+            switch (typecode) {
+            case 'z':
+                typeId = JBOOLEAN_ID;
+                arrayObj = (*env)->NewBooleanArray(env, (jsize) size);
+                break;
+            case 'b':
+                typeId = JBYTE_ID;
+                arrayObj = (*env)->NewByteArray(env, (jsize) size);
+                break;
+            case 'c':
+                typeId = JCHAR_ID;
+                arrayObj = (*env)->NewCharArray(env, (jsize) size);
+                break;
+            case 's':
+                typeId = JSHORT_ID;
+                arrayObj = (*env)->NewShortArray(env, (jsize) size);
+                break;
+            case 'i':
+                typeId = JINT_ID;
+                arrayObj = (*env)->NewIntArray(env, (jsize) size);
+                break;
+            case 'j':
+                typeId = JLONG_ID;
+                arrayObj = (*env)->NewLongArray(env, (jsize) size);
+                break;
+            case 'f':
+                typeId = JFLOAT_ID;
+                arrayObj = (*env)->NewFloatArray(env, (jsize) size);
+                break;
+            case 'd':
+                typeId = JDOUBLE_ID;
+                arrayObj = (*env)->NewDoubleArray(env, (jsize) size);
+                break;
+            } // switch
+            if (!arrayObj) {
+                PyErr_SetString(PyExc_ValueError,
+                                "bad typecode (must be z, b, c, s, i, j, f or d)");
+                return NULL;
+            }
         } else {
             PyErr_SetString(PyExc_ValueError, "Unknown arg type: expected "
                             "one of: J<foo>_ID, pyjobject, jarray");
@@ -178,7 +257,7 @@ PyObject* pyjarray_new_v(PyObject *isnull, PyObject *args)
         return NULL;
     }
 
-    if (!arrayObj || typeId < -1 || size < -1) {
+    if (!arrayObj || typeId < 0 || size < 0) {
         PyErr_SetString(PyExc_ValueError, "Unknown type.");
         return NULL;
     }
@@ -251,8 +330,8 @@ static int pyjarray_init(JNIEnv *env,
             long  v  = 0;
             jint *ar = (jint *) pyarray->pinnedArray;
 
-            if (value && PyInt_Check(value)) {
-                v = (long) PyInt_AS_LONG(value);
+            if (value && PyLong_Check(value)) {
+                v = (long) PyLong_AsLongLong(value);
             }
 
             for (i = 0; i < pyarray->length; i++) {
@@ -267,9 +346,9 @@ static int pyjarray_init(JNIEnv *env,
             long  v = 0;
             const char *val;
             jchar *ar = (jchar *) pyarray->pinnedArray;
-            if (!value || !PyString_Check(value)) {
-                if (value && PyInt_Check(value)) {
-                    v = (long) PyInt_AS_LONG(value);
+            if (!value || !PyUnicode_Check(value)) {
+                if (value && PyLong_Check(value)) {
+                    v = (long) PyLong_AsLongLong(value);
                 }
 
                 for (i = 0; i < pyarray->length; i++) {
@@ -279,7 +358,7 @@ static int pyjarray_init(JNIEnv *env,
                 // it's a string, set the elements
                 // we won't throw an error for index, length problems. just deal...
 
-                val = PyString_AS_STRING(value);
+                val = PyUnicode_AsUTF8(value);
                 for (i = 0; i < pyarray->length && val[i] != '\0'; i++) {
                     ar[i] = (jchar) val[i];
                 }
@@ -293,8 +372,8 @@ static int pyjarray_init(JNIEnv *env,
             long   v  = 0;
             jbyte *ar = (jbyte *) pyarray->pinnedArray;
 
-            if (value && PyInt_Check(value)) {
-                v = (long) PyInt_AS_LONG(value);
+            if (value && PyLong_Check(value)) {
+                v = (long) PyLong_AsLongLong(value);
             }
 
             for (i = 0; i < pyarray->length; i++) {
@@ -311,12 +390,8 @@ static int pyjarray_init(JNIEnv *env,
 
             if (!value)
                 ;
-            else {
-                if (PyLong_Check(value)) {
-                    v = PyLong_AsLongLong(value);
-                } else if (PyInt_Check(value)) {
-                    v = PyInt_AS_LONG(value);
-                }
+            else if (PyLong_Check(value)) {
+                v = PyLong_AsLongLong(value);
             }
 
             for (i = 0; i < pyarray->length; i++) {
@@ -331,8 +406,8 @@ static int pyjarray_init(JNIEnv *env,
             long      v  = 0;
             jboolean *ar = (jboolean *) pyarray->pinnedArray;
 
-            if (value && PyInt_Check(value)) {
-                v = (long) PyInt_AS_LONG(value);
+            if (value && PyLong_Check(value)) {
+                v = (long) PyLong_AsLongLong(value);
             }
 
             for (i = 0; i < pyarray->length; i++) {
@@ -367,8 +442,8 @@ static int pyjarray_init(JNIEnv *env,
             long    v  = 0;
             jshort *ar = (jshort *) pyarray->pinnedArray;
 
-            if (value && PyInt_Check(value)) {
-                v  = (long) PyInt_AS_LONG(value);
+            if (value && PyLong_Check(value)) {
+                v  = (long) PyLong_AsLongLong(value);
             }
 
             for (i = 0; i < pyarray->length; i++) {
@@ -606,7 +681,7 @@ static int pyjarray_setitem(PyJArrayObject *self,
         if (newitem == Py_None)
             ; // setting NULL
         else {
-            if (!PyString_Check(newitem)) {
+            if (!PyUnicode_Check(newitem)) {
                 PyErr_SetString(PyExc_TypeError, "Expected string.");
                 return -1;
             }
@@ -677,28 +752,28 @@ static int pyjarray_setitem(PyJArrayObject *self,
     switch (self->componentType) {
 
     case JINT_ID:
-        if (!PyInt_Check(newitem)) {
+        if (!PyLong_Check(newitem)) {
             PyErr_SetString(PyExc_TypeError, "Expected int.");
             return -1;
         }
 
-        ((jint *) self->pinnedArray)[pos] = (jint) PyInt_AS_LONG(newitem);
+        ((jint *) self->pinnedArray)[pos] = (jint) PyLong_AsLongLong(newitem);
         return 0; /* success */
 
     case JBYTE_ID:
-        if (!PyInt_Check(newitem)) {
+        if (!PyLong_Check(newitem)) {
             PyErr_SetString(PyExc_TypeError, "Expected byte.");
             return -1;
         }
 
-        ((jbyte *) self->pinnedArray)[pos] = (jbyte) PyInt_AS_LONG(newitem);
+        ((jbyte *) self->pinnedArray)[pos] = (jbyte) PyLong_AsLongLong(newitem);
         return 0; /* success */
 
     case JCHAR_ID:
-        if (PyInt_Check(newitem)) {
-            ((jchar *) self->pinnedArray)[pos] = (jchar) PyInt_AS_LONG(newitem);
-        } else if (PyString_Check(newitem) && PyString_GET_SIZE(newitem) == 1) {
-            const char *val = PyString_AS_STRING(newitem);
+        if (PyLong_Check(newitem)) {
+            ((jchar *) self->pinnedArray)[pos] = (jchar) PyLong_AsLongLong(newitem);
+        } else if (PyUnicode_Check(newitem) && PyUnicode_GET_LENGTH(newitem) == 1) {
+            const char *val = PyUnicode_AsUTF8(newitem);
             ((jchar *) self->pinnedArray)[pos] = (jchar) val[0];
         } else {
             PyErr_SetString(PyExc_TypeError, "Expected char.");
@@ -717,12 +792,12 @@ static int pyjarray_setitem(PyJArrayObject *self,
         return 0; /* success */
 
     case JBOOLEAN_ID:
-        if (!PyInt_Check(newitem)) {
+        if (!PyLong_Check(newitem)) {
             PyErr_SetString(PyExc_TypeError, "Expected boolean.");
             return -1;
         }
 
-        if (PyInt_AS_LONG(newitem)) {
+        if (PyLong_AsLongLong(newitem)) {
             ((jboolean *) self->pinnedArray)[pos] = JNI_TRUE;
         } else {
             ((jboolean *) self->pinnedArray)[pos] = JNI_FALSE;
@@ -741,13 +816,13 @@ static int pyjarray_setitem(PyJArrayObject *self,
         return 0; /* success */
 
     case JSHORT_ID:
-        if (!PyInt_Check(newitem)) {
+        if (!PyLong_Check(newitem)) {
             PyErr_SetString(PyExc_TypeError, "Expected int.");
             return -1;
         }
 
         ((jshort *) self->pinnedArray)[pos] =
-            (jshort) PyInt_AS_LONG(newitem);
+            (jshort) PyLong_AsLongLong(newitem);
         return 0; /* success */
 
     case JFLOAT_ID:
@@ -900,7 +975,7 @@ static int pyjarray_index(PyJArrayObject *self, PyObject *el)
     case JSTRING_ID: {
         int i, ret = 0;
 
-        if (el != Py_None && !PyString_Check(el)) {
+        if (el != Py_None && !PyUnicode_Check(el)) {
             PyErr_SetString(PyExc_TypeError, "Expected str.");
             return -1;
         }
@@ -1020,12 +1095,12 @@ static int pyjarray_index(PyJArrayObject *self, PyObject *el)
         int       i;
         jboolean  v;
 
-        if (!PyInt_Check(el)) {
+        if (!PyLong_Check(el)) {
             PyErr_SetString(PyExc_TypeError, "Expected boolean.");
             return -1;
         }
 
-        if (PyInt_AS_LONG(el)) {
+        if (PyLong_AsLongLong(el)) {
             v = JNI_TRUE;
         } else {
             v = JNI_FALSE;
@@ -1045,12 +1120,12 @@ static int pyjarray_index(PyJArrayObject *self, PyObject *el)
         int     i;
         jshort  v;
 
-        if (!PyInt_Check(el)) {
+        if (!PyLong_Check(el)) {
             PyErr_SetString(PyExc_TypeError, "Expected int (short).");
             return -1;
         }
 
-        v = (jshort) PyInt_AS_LONG(el);
+        v = (jshort) PyLong_AsLongLong(el);
         for (i = 0; i < self->length; i++) {
             if (v == ar[i]) {
                 return i;
@@ -1065,12 +1140,12 @@ static int pyjarray_index(PyJArrayObject *self, PyObject *el)
         int   i;
         jint  v;
 
-        if (!PyInt_Check(el)) {
+        if (!PyLong_Check(el)) {
             PyErr_SetString(PyExc_TypeError, "Expected int.");
             return -1;
         }
 
-        v = (jint) PyInt_AS_LONG(el);
+        v = (jint) PyLong_AsLongLong(el);
         for (i = 0; i < self->length; i++) {
             if (v == ar[i]) {
                 return i;
@@ -1085,12 +1160,12 @@ static int pyjarray_index(PyJArrayObject *self, PyObject *el)
         int    i;
         jbyte  v;
 
-        if (!PyInt_Check(el)) {
+        if (!PyLong_Check(el)) {
             PyErr_SetString(PyExc_TypeError, "Expected byte.");
             return -1;
         }
 
-        v = (jbyte) PyInt_AS_LONG(el);
+        v = (jbyte) PyLong_AsLongLong(el);
         for (i = 0; i < self->length; i++) {
             if (v == ar[i]) {
                 return i;
@@ -1105,10 +1180,10 @@ static int pyjarray_index(PyJArrayObject *self, PyObject *el)
         int    i;
         jchar  v;
 
-        if (PyInt_Check(el)) {
-            v = (jchar) PyInt_AS_LONG(el);
-        } else if (PyString_Check(el) && PyString_GET_SIZE(el) == 1) {
-            const char *val = PyString_AS_STRING(el);
+        if (PyLong_Check(el)) {
+            v = (jchar) PyLong_AsLongLong(el);
+        } else if (PyUnicode_Check(el) && PyUnicode_GET_LENGTH(el) == 1) {
+            const char *val = PyUnicode_AsUTF8(el);
             v = (jchar) val[0];
         } else {
             PyErr_SetString(PyExc_TypeError, "Expected char.");
@@ -1208,7 +1283,7 @@ static PyObject* listindex(PyJArrayObject *self, PyObject *args)
     }
 
     if (pos >= 0) {
-        return PyInt_FromLong((long) pos);
+        return PyLong_FromLongLong((long) pos);
     }
 
     PyErr_SetString(PyExc_ValueError, "list.index(x): x not in array");
@@ -1475,13 +1550,7 @@ static PyObject* pyjarray_slice(PyObject *_self, Py_ssize_t ilow,
 // shamelessly taken from listobject.c
 static PyObject* pyjarray_subscript(PyJArrayObject *self, PyObject *item)
 {
-    if (PyInt_Check(item)) {
-        long i = (long) PyInt_AS_LONG(item);
-        if (i < 0) {
-            i += self->length;
-        }
-        return pyjarray_item(self, (Py_ssize_t) i);
-    } else if (PyLong_Check(item)) {
+    if (PyLong_Check(item)) {
         long i = PyLong_AsLong(item);
         if (i == -1 && PyErr_Occurred()) {
             return NULL;
@@ -1492,25 +1561,11 @@ static PyObject* pyjarray_subscript(PyJArrayObject *self, PyObject *item)
         return pyjarray_item(self, (Py_ssize_t) i);
     } else if (PySlice_Check(item)) {
         Py_ssize_t start, stop, step, slicelength;
-
-#if PY_MAJOR_VERSION >= 3
         if (PySlice_GetIndicesEx(item, pyjarray_length((PyObject*) self), &start, &stop,
                                  &step, &slicelength) < 0) {
             // error will already be set
             return NULL;
         }
-#else
-        /*
-         * This silences a compile warning on PySlice_GetIndicesEx by casting
-         * item.  Python fixed the method signature in 3.2 to take item as a
-         * PyObject*
-         */
-        if (PySlice_GetIndicesEx((PySliceObject *) item,
-                                 pyjarray_length((PyObject*) self), &start, &stop, &step, &slicelength) < 0) {
-            // error will already be set
-            return NULL;
-        }
-#endif
 
         if (slicelength <= 0) {
             return pyjarray_slice((PyObject*) self, 0, 0);
@@ -1531,30 +1586,10 @@ static PyObject* pyjarray_subscript(PyJArrayObject *self, PyObject *item)
 static PyObject* pyjarray_str(PyJArrayObject *self)
 {
     PyObject *ret;
-#if PY_MAJOR_VERSION >= 3
     JNIEnv   *env = pyembed_get_env();
 
     ret = jobject_As_PyString(env, self->object);
     return ret;
-#else
-    // retained to not break former behavior
-    if (!self->pinnedArray) {
-        PyErr_SetString(PyExc_RuntimeError, "No pinned array.");
-        return NULL;
-    }
-
-    switch (self->componentType) {
-    case JBYTE_ID:
-        ret = PyBytes_FromStringAndSize((const char *) self->pinnedArray,
-                                        self->length);
-        return ret;
-
-    default:
-        PyErr_SetString(PyExc_TypeError,
-                        "Unsupported type for str operation.");
-        return NULL;
-    }
-#endif
 }
 
 
@@ -1635,8 +1670,7 @@ PyTypeObject PyJArray_Type = {
     0,                                        /* tp_getattro */
     0,                                        /* tp_setattro */
     0,                                        /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT |
-    Py_TPFLAGS_HAVE_ITER,                     /* tp_flags */
+    Py_TPFLAGS_DEFAULT,                       /* tp_flags */
     list_doc,                                 /* tp_doc */
     0,                                        /* tp_traverse */
     0,                                        /* tp_clear */
@@ -1765,8 +1799,7 @@ PyTypeObject PyJArrayIter_Type = {
     (getattrofunc) pyjarrayiter_dealloc,      /* tp_getattro */
     0,                                        /* tp_setattro */
     0,                                        /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT |
-    Py_TPFLAGS_HAVE_ITER,                     /* tp_flags */
+    Py_TPFLAGS_DEFAULT,                       /* tp_flags */
     0,                                        /* tp_doc */
     0,                                        /* tp_traverse */
     0,                                        /* tp_clear */
