@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2004-2019 JEP AUTHORS.
+   Copyright (c) 2004-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -179,7 +179,7 @@ int PyJMethod_CheckArguments(PyJMethodObject* method, JNIEnv *env,
                                 method->parameters, parampos);
 
         if (process_java_exception(env) || !paramType) {
-            match = 0;
+            matchTotal = -1;
             break;
         }
 
@@ -188,11 +188,7 @@ int PyJMethod_CheckArguments(PyJMethodObject* method, JNIEnv *env,
         match = pyarg_matches_jtype(env, param, paramType, paramTypeId);
         (*env)->DeleteLocalRef(env, paramType);
         if (PyErr_Occurred()) {
-            match = 0;
-            break;
-        }
-
-        if (!match) {
+            matchTotal = -1;
             break;
         }
         matchTotal += match;
@@ -225,7 +221,7 @@ static PyObject* pyjmethod_call(PyJMethodObject *self,
     /* if params includes pyjarray instance */
     int            foundArray       = 0;
 
-    if (keywords != NULL) {
+    if (keywords != NULL && PyDict_Size(keywords) > 0) {
         PyErr_Format(PyExc_RuntimeError, "Keywords are not supported.");
         return NULL;
     }
