@@ -1,7 +1,7 @@
 /*
    jep - Java Embedded Python
 
-   Copyright (c) 2004-2019 JEP AUTHORS.
+   Copyright (c) 2004-2021 JEP AUTHORS.
 
    This file is licensed under the the zlib/libpng License.
 
@@ -36,10 +36,10 @@
 /* Set the object attributes from the cache */
 static int pyjobject_init(JNIEnv *env, PyJObject *pyjob)
 {
-
     jstring className     = NULL;
     PyObject *pyClassName = NULL;
-    JepThread *jepThread  = NULL;
+    PyObject* modjep      = NULL;
+    PyObject* fqnToPyJAttrs = NULL;
     PyObject *cachedAttrs = NULL;
 
     if ((*env)->PushLocalFrame(env, JLOCAL_REFS) != 0) {
@@ -81,15 +81,16 @@ static int pyjobject_init(JNIEnv *env, PyJObject *pyjob)
      * synchronized and multiple threads will not alter the dictionary at the
      * same time.
      */
-    jepThread = pyembed_get_jepthread();
-    if (jepThread == NULL) {
+    modjep = PyImport_ImportModule("_jep");
+    if (!modjep) {
         goto EXIT_ERROR;
     }
-    if (jepThread->fqnToPyJAttrs == NULL) {
-        jepThread->fqnToPyJAttrs = PyDict_New();
+    fqnToPyJAttrs = PyObject_GetAttrString(modjep, "__javaAttributeCache__");
+    if (!fqnToPyJAttrs) {
+        goto EXIT_ERROR;
     }
 
-    cachedAttrs = PyDict_GetItem(jepThread->fqnToPyJAttrs, pyClassName);
+    cachedAttrs = PyDict_GetItem(fqnToPyJAttrs, pyClassName);
     if (cachedAttrs == NULL) {
         int i, len = 0;
         jobjectArray methodArray = NULL;
@@ -133,7 +134,7 @@ static int pyjobject_init(JNIEnv *env, PyJObject *pyjob)
              * so, turn it into a PyJMultiMethod or add it to the existing
              * PyJMultiMethod.
              */
-            if (pymethod->pyMethodName && PyString_Check(pymethod->pyMethodName)) {
+            if (pymethod->pyMethodName && PyUnicode_Check(pymethod->pyMethodName)) {
                 PyObject* cached = PyDict_GetItem(cachedAttrs, pymethod->pyMethodName);
                 if (cached == NULL) {
                     if (PyDict_SetItem(cachedAttrs, pymethod->pyMethodName,
@@ -174,7 +175,7 @@ static int pyjobject_init(JNIEnv *env, PyJObject *pyjob)
                 continue;
             }
 
-            if (pyjfield->pyFieldName && PyString_Check(pyjfield->pyFieldName)) {
+            if (pyjfield->pyFieldName && PyUnicode_Check(pyjfield->pyFieldName)) {
                 if (PyDict_SetItem(cachedAttrs, pyjfield->pyFieldName,
                                    (PyObject*) pyjfield) != 0) {
                     goto EXIT_ERROR;
@@ -186,7 +187,7 @@ static int pyjobject_init(JNIEnv *env, PyJObject *pyjob)
         }
         (*env)->DeleteLocalRef(env, fieldArray);
 
-        PyDict_SetItem(jepThread->fqnToPyJAttrs, pyClassName, cachedAttrs);
+        PyDict_SetItem(fqnToPyJAttrs, pyClassName, cachedAttrs);
         Py_DECREF(cachedAttrs); // fqnToPyJAttrs will hold the reference
     } // end of setting up cache for this Java Class
 
@@ -198,11 +199,15 @@ static int pyjobject_init(JNIEnv *env, PyJObject *pyjob)
         pyjob->attr = PyDict_Copy(cachedAttrs);
     }
 
+    Py_XDECREF(modjep);
+    Py_XDECREF(fqnToPyJAttrs);
     (*env)->PopLocalFrame(env, NULL);
     return 1;
 
 
 EXIT_ERROR:
+    Py_XDECREF(modjep);
+    Py_XDECREF(fqnToPyJAttrs);
     (*env)->PopLocalFrame(env, NULL);
     return 0;
 }
@@ -211,7 +216,7 @@ EXIT_ERROR:
 PyObject* PyJObject_New(JNIEnv *env, PyTypeObject* type, jobject obj,
                         jclass class)
 {
-    PyJObject *pyjob = PyObject_NEW(PyJObject, type);
+    PyJObject *pyjob = (PyJObject*) PyType_GenericAlloc(type, 0);
 
     if (obj) {
         pyjob->object = (*env)->NewGlobalRef(env, obj);
@@ -250,8 +255,7 @@ static void pyjobject_dealloc(PyJObject *self)
 
     Py_CLEAR(self->attr);
     Py_CLEAR(self->javaClassName);
-
-    PyObject_Del(self);
+    Py_TYPE((PyObject*) self)->tp_free((PyObject*) self);
 #endif
 }
 
@@ -272,6 +276,48 @@ static PyObject* pyjobject_str(PyJObject *self)
     return pyres;
 }
 
+// call __repr__() on jobject. returns null on error.
+// expected to return new reference.
+static PyObject* pyjobject_repr(PyJObject *self)
+{
+    PyObject   *pyres   = NULL;
+    JNIEnv     *env;
+    jmethodID  __repr__ = 0;
+    jstring    jrepr    = NULL;
+
+    //check if __repr__ method exists on java object
+    if (self->object && self->clazz) {
+        PyObject *repr = PyDict_GetItemString(self->attr, "__repr__");
+        if (repr) {
+            env   = pyembed_get_env();
+
+            Py_BEGIN_ALLOW_THREADS
+            if (JNI_METHOD(__repr__, env, self->clazz, "__repr__", "()Ljava/lang/String;")) {
+                jrepr = (jstring) (*env)->CallObjectMethod(env, self->object, __repr__);
+            } else {
+                //method does not exist, clear exception
+                if ((*env)->ExceptionCheck(env)) {
+                    (*env)->ExceptionClear(env);
+                }
+            }
+            Py_END_ALLOW_THREADS
+            if (process_java_exception(env)) {
+                return NULL;
+            }
+            if (jrepr != NULL) {
+                pyres = jstring_As_PyString(env, jrepr);
+            }
+            (*env)->DeleteLocalRef(env, jrepr);
+        }
+    }
+
+    if (pyres == NULL) {
+        //default (see: PyObject_Repr)
+        pyres =  PyUnicode_FromFormat("<%s object at %p>", Py_TYPE(self)->tp_name, self);
+    }
+    return pyres;
+}
+
 
 static PyObject* pyjobject_richcompare(PyJObject *self,
                                        PyObject *_other,
@@ -337,19 +383,15 @@ static PyObject* pyjobject_richcompare(PyJObject *self,
              * raise a TypeError.
              */
             jint result;
-#if PY_MAJOR_VERSION >= 3
             jthrowable exc;
-#endif
-
             if (!(*env)->IsInstanceOf(env, self->object, JCOMPARABLE_TYPE)) {
-                const char* jname = PyString_AsString(self->javaClassName);
+                const char* jname = PyUnicode_AsUTF8(self->javaClassName);
                 PyErr_Format(PyExc_TypeError, "Invalid comparison operation for Java type %s",
                              jname);
                 return NULL;
             }
 
             result = java_lang_Comparable_compareTo(env, target, other_target);
-#if PY_MAJOR_VERSION >= 3
             exc = (*env)->ExceptionOccurred(env);
             if (exc != NULL) {
                 if ((*env)->IsInstanceOf(env, exc, CLASSCAST_EXC_TYPE)) {
@@ -365,7 +407,6 @@ static PyObject* pyjobject_richcompare(PyJObject *self,
                     return Py_NotImplemented;
                 }
             }
-#endif
             if (process_java_exception(env)) {
                 return NULL;
             }
@@ -410,12 +451,7 @@ static PyObject* pyjobject_getattro(PyObject *obj, PyObject *name)
          * TODO Should not bind non-static methods to pyjclass objects, but not
          * sure yet how to handle multimethods and static methods.
          */
-#if PY_MAJOR_VERSION >= 3
         PyObject* wrapper = PyMethod_New(ret, (PyObject*) obj);
-#else
-        PyObject* wrapper = PyMethod_New(ret, (PyObject*) obj,
-                                         (PyObject*) Py_TYPE(obj));
-#endif
         Py_DECREF(ret);
         return wrapper;
     } else if (PyJField_Check(ret)) {
@@ -445,18 +481,18 @@ static int pyjobject_setattro(PyJObject *obj, PyObject *name, PyObject *v)
 
     if (cur == NULL) {
         PyErr_Format(PyExc_AttributeError, "'%s' object has no attribute '%s'.",
-                     PyString_AsString(obj->javaClassName), PyString_AsString(name));
+                     PyUnicode_AsUTF8(obj->javaClassName), PyUnicode_AsUTF8(name));
         return -1;
     }
 
     if (!PyJField_Check(cur)) {
         if (PyJMethod_Check(cur) || PyJMultiMethod_Check(cur)) {
             PyErr_Format(PyExc_AttributeError, "'%s' object cannot assign to method '%s'.",
-                         PyString_AsString(obj->javaClassName), PyString_AsString(name));
+                         PyUnicode_AsUTF8(obj->javaClassName), PyUnicode_AsUTF8(name));
         } else {
             PyErr_Format(PyExc_AttributeError,
                          "'%s' object cannot assign to attribute '%s'.",
-                         PyString_AsString(obj->javaClassName), PyString_AsString(name));
+                         PyUnicode_AsUTF8(obj->javaClassName), PyUnicode_AsUTF8(name));
         }
         return -1;
     }
@@ -529,7 +565,7 @@ static PyMemberDef pyjobject_members[] = {
 
 PyTypeObject PyJObject_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "jep.PyJObject",                          /* tp_name */
+    "java.lang.Object",                       /* tp_name */
     sizeof(PyJObject),                        /* tp_basicsize */
     0,                                        /* tp_itemsize */
     (destructor) pyjobject_dealloc,           /* tp_dealloc */
@@ -537,7 +573,7 @@ PyTypeObject PyJObject_Type = {
     0,                                        /* tp_getattr */
     0,                                        /* tp_setattr */
     0,                                        /* tp_compare */
-    0,                                        /* tp_repr */
+    (reprfunc) pyjobject_repr,                /* tp_repr */
     0,                                        /* tp_as_number */
     0,                                        /* tp_as_sequence */
     0,                                        /* tp_as_mapping */
@@ -549,7 +585,7 @@ PyTypeObject PyJObject_Type = {
     0,                                        /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT |
     Py_TPFLAGS_BASETYPE,                      /* tp_flags */
-    "jobject",                                /* tp_doc */
+    "Jep java.lang.Object",                   /* tp_doc */
     0,                                        /* tp_traverse */
     0,                                        /* tp_clear */
     (richcmpfunc) pyjobject_richcompare,      /* tp_richcompare */
