@@ -1,5 +1,6 @@
 import unittest
 import sys
+import jep
 from jep_pipe import jep_pipe
 from jep_pipe import build_python_process_cmd
 
@@ -16,7 +17,13 @@ class TestIterators(unittest.TestCase):
         for i in x:
             self.assertIn("a", i)
 
-    @unittest.skipIf(sys.platform.startswith("win"), "subprocess complications on Windows")
     def test_iter_itr_crash(self):
         jep_pipe(build_python_process_cmd(
             'src/test/python/subprocess/iter_itr_crash.py'))
+
+    def test_iteratorable(self):
+        TestIteratorable = jep.findClass('jep.test.TestIteratorable')
+        iterator = TestIteratorable()
+        iterable = TestIteratorable()
+        for item in iterable:
+            self.assertEquals(item, next(iterator))
