@@ -1,6 +1,8 @@
 import unittest
 from java.lang import Object
+import jep
 
+TestPyJObject = jep.findClass('jep.test.TestPyJObject')
 
 class TestObject(unittest.TestCase):
 
@@ -12,6 +14,11 @@ class TestObject(unittest.TestCase):
         o = Object()
         self.assertIn('java.lang.Object@', str(o))
 
+    def test_repr(self):
+        self.assertEquals(repr(TestPyJObject.ReprClass()), "ReprClass")
+        self.assertEquals(repr(TestPyJObject.ReprSubClass()), "ReprSubClass")
+        self.assertIn("<java.lang.Object object at", repr(Object()))
+
     def test_del_throws_exception(self):
         o = Object()
         with self.assertRaises(TypeError):
